@@ -22,7 +22,7 @@
 #include "timeout.h"
 
 static char *const _id =
-"$Id: checkpc.c,v 3.1 1996/05/22 03:36:21 papowell Exp papowell $";
+"$Id: checkpc.c,v 3.3 1996/09/09 14:24:41 papowell Exp papowell $";
 
 char checkpc_optstr[] = "ac:flp:rst:A:CD:PT:V";
 
@@ -75,7 +75,6 @@ int main( int argc, char *argv[] )
 
 
     /* scan the argument list for a 'Debug' value */
-	Setup_waitpid();
     Get_debug_parm( argc, argv, checkpc_optstr, debug_vars );
 	while( (c = Getopt( argc, argv, checkpc_optstr ) ) != EOF ){
 		switch( c ){
@@ -216,7 +215,14 @@ int main( int argc, char *argv[] )
 		logDebug( "Warning: no LPD lockfile" );
 	} else {
 		logDebug( "LPD lockfile '%s'", Lockfile );
-		Make_write_file( SDpathname, 0, Lockfile, (char *)0 );
+		s = strrchr( Lockfile, '/' );
+		if( s ){
+			struct dpathname newpath;
+			*s++ = 0;
+			Init_path( &newpath, Lockfile );
+			Check_spool_dir( &newpath, Fix );
+			Make_write_file( &newpath, 0, s, (char *)0 );
+		}
 	}
 
 	if( All_list ){
@@ -443,7 +449,7 @@ void Make_write_file( struct dpathname *dpathname,
 		s = Add2_path( dpathname, name, printer );
 	}
 
-	DEBUG8("Make_write_file '%s'", s );
+	DEBUG1("Make_write_file '%s'", s );
 	if( Verbose || Debug > 0 ){
 		logDebug( "  checking file '%s'", s );
 	}
