@@ -23,7 +23,7 @@ We are going to simulate the LPD daemon using the basic facilities.
 
 */
 static char *const _id =
-"$Id: lpd.c,v 3.1 1996/07/20 02:42:11 papowell Exp papowell $";
+"$Id: lpd.c,v 3.3 1996/08/31 21:11:58 papowell Exp papowell $";
 
 #include "lpd.h"
 #include "printcap.h"
@@ -126,8 +126,6 @@ int main(int argc, char *argv[], char *envp[])
 	(void) plp_signal (SIGQUIT, cleanup);
 	(void) plp_signal (SIGTERM, cleanup);
 
-	/* prepare to catch dead children */
-	Setup_waitpid();
 
 	/* Get configuration file information */
 	Parsebuffer( "default configuration",Default_configuration,
@@ -266,6 +264,8 @@ int main(int argc, char *argv[], char *envp[])
 		Process_jobs( 0, b, sizeof(b) );
 		exit(0);
 	}
+	/* open a connection to logger */
+	send_to_logger( 0 );
 
 	/*
 	 * clean out the current queues
