@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: lpd_control.c,v 3.1 1996/06/30 17:12:44 papowell Exp $";
+"$Id: lpd_control.c,v 3.4 1996/08/31 21:11:58 papowell Exp papowell $";
 
 #include "lpd.h"
 #include "printcap.h"
@@ -268,16 +268,30 @@ void Do_printer_work( char *user, int action, int *socket,
 			}
 		} else {
 			for( i = 0; i < Printcapfile.pcs.count; ++i ){
+				char *pn, *npn;
+				int j;
 				pc = (void *)Printcapfile.pcs.list;
 				pc = &pc[i];
-				Printer = pc->pcf->lines.list[pc->name];
+				pn = Printer = pc->pcf->lines.list[pc->name];
 				DEBUG4( "Job_control: checking printcap entry '%s'",  Printer );
+				if( pn == 0 || *pn == 0 ){
+					continue;
+				}
 
 				/* this is a printcap entry with spool directory -
 					we need to check it out
 				 */
 				Do_queue_server( user, action, socket,
 					tokencount-4, &tokens[4], error, errorlen);
+				for( j = i + 1; j < Printcapfile.pcs.count; ++j ){
+					pc = (void *)Printcapfile.pcs.list;
+					pc = &pc[j];
+					npn = pc->pcf->lines.list[pc->name];
+					if( npn && strcmp(npn, pn) == 0 ){
+						DEBUG4( "Job_control: deleting entry '%s'",  npn );
+						*npn = 0;
+					}
+				}
 			}
 		}
 	}
@@ -582,6 +596,12 @@ int Do_control_file( char *user, int action, int *socket,
 	char msg[SMALLBUFFER];			/* message field */
 	struct control_file *cfp, **cfpp;	/* control files */
 	int select;						/* select this file */
+	struct token token;
+	struct destination *destination;
+
+	/* set up default token */
+	token.start = user;
+	token.length = strlen( user );
 
 	/* get the spool entries */
 	Scan_queue( 0 );
@@ -602,37 +622,97 @@ int Do_control_file( char *user, int action, int *socket,
 		DEBUG8("Do_control_file: checking '%s'", cfp->name );
 
 		select = 1;
+		destination = 0;
+
+next_destination:
 		if( tokencount > 0 ){
 			for( j = 0; select && j < tokencount; ++j ){
-				select = patselect( &tokens[j], cfp );
+				select = patselect( &tokens[j], cfp, &destination );
 			}
 		} else {
-			struct token token;
-			token.start = user;
-			token.length = strlen( user );
-			select = patselect( &token, cfp );
+			select = patselect( &token, cfp, &destination );
 		}
 		if( !select ) continue;
 
 		/* now we get the user name and IP address */
 
-		DEBUG8("Do_control_file: selected '%s'", cfp->name );
+		DEBUG8("Do_control_file: selected '%s', id '%s', destination '%s'",
+			cfp->name, cfp->identifier, destination->destination );
 		/* we report this job being selected */
-		plp_snprintf( msg, sizeof(msg), "selected '%s'", cfp->name );
+		plp_snprintf( msg, sizeof(msg), "selected '%s'", cfp->identifier );
+		if( destination ){
+			plp_snprintf( msg, sizeof(msg), "selected '%s'", destination->identifier );
+		}
 		status = Link_send( ShortRemote, socket, Send_timeout,
 			0x00, msg, '\n', 0 );
 		switch( action ){
-		case HOLD: cfp->hold_time = time( (void *)0 ); break;
-		case RELEASE: cfp->hold_time = 0; break;
-		case TOPQ: cfp->priority_time = time( (void *)0 ); break;
+		case HOLD:
+			if( destination ){
+				destination->hold = time( (void *)0 );
+			} else {
+				cfp->hold_time = time( (void *)0 );
+				destination = (void *)cfp->destination_list.list;
+				for( j =0; j < cfp->destination_list.count; ++j ){
+					destination[j].hold = cfp->hold_time;
+				}
+				destination = 0;
+			}
+			setmessage( cfp, "TRACE", "%s@%s: job held", Printer, FQDNHost );
+			break;
+		case TOPQ: cfp->priority_time = time( (void *)0 );
+			if( destination ){
+				cfp->hold_time = 0;
+				destination->hold = 0;
+			} else {
+				cfp->hold_time = 0;
+				destination = (void *)cfp->destination_list.list;
+				for( j =0; j < cfp->destination_list.count; ++j ){
+					destination[j].hold = 0;
+				}
+				destination = 0;
+			}
+			setmessage( cfp, "TRACE", "%s@%s: job topq", Printer, FQDNHost );
+			break;
 		case MOVE:
-				strcpy( cfp->redirect, Redirect_str );
-				/* and we update the priority to put it at head of queue */
-				cfp->priority_time = time( (void *)0 ); break;
-				break;
+			strcpy( cfp->redirect, Redirect_str );
+			/* and we update the priority to put it at head of queue */
+			cfp->priority_time = time( (void *)0 );
+			cfp->hold_time = 0;
+			setmessage( cfp, "TRACE", "%s@%s: job moved", Printer, FQDNHost );
+			break;
+		case RELEASE:
+			cfp->hold_time = 0;
+			if( destination ){
+				destination->hold = 0;
+			} else {
+				destination = (void *)cfp->destination_list.list;
+				for( j =0; j < cfp->destination_list.count; ++j ){
+					destination[j].hold = 0;
+				}
+				destination = 0;
+			}
+			setmessage( cfp, "TRACE", "%s@%s: job released", Printer, FQDNHost );
+			break;
 		}
+		cfp->done_time = 0;
 		cfp->error[0] = 0;
+		if( destination ){
+			destination->error[0] = 0;
+			destination->done = 0;
+		} else {
+			destination = (void *)cfp->destination_list.list;
+			for( j =0; j < cfp->destination_list.count; ++j ){
+				destination[j].done = 0;
+				destination[j].error[0] = 0;
+			}
+			destination = 0;
+		}
 		Set_job_control( cfp );
+		if( tokencount <= 0 ){
+			DEBUG8("Do_control_file: finished '%s'", cfp->name );
+			break;
+		}
+		if( destination ) goto next_destination;
 	}
 	return( 0 );
 }
@@ -690,12 +770,14 @@ int Do_control_status( char *user, int action, int *socket,
 	int fd;
 	struct stat statb;
 	char *path;
+	int cnt, hold_cnt;
 
 	/* get the spool entries */
 	Scan_queue( 0 );
 	DEBUG8("Do_control_status: total files %d, tokencount %d",
 		C_files_list.count, tokencount );
 
+	job_count( &hold_cnt, &cnt );
 
 	/* now check to see if there is a server and unspooler process active */
 	path = Add_path( CDpathname, Printer );
@@ -736,7 +818,7 @@ int Do_control_status( char *user, int action, int *socket,
 		len = strlen(pr);
 		plp_snprintf( pr+len, sizeof(pr)-len, " autohold" );
 	}
-	plp_snprintf( count, sizeof(count), "%d", C_files_list.count );
+	plp_snprintf( count, sizeof(count), "%d", cnt );
 	strcpy( server, "none" );
 	strcpy( spooler, "none" );
 	if( serverpid ) plp_snprintf( server, sizeof(server),"%d",serverpid );
@@ -753,8 +835,7 @@ int Do_control_status( char *user, int action, int *socket,
 		Printing_disabled? "disabled" : "enabled",
 		Spooling_disabled? "disabled" : "enabled",
 		count, server, spooler, forward, Control_debug?Control_debug:"" );
-
-
+	trunc_str( msg );
 	(void)Link_send( ShortRemote, socket, Send_timeout,
 		0x00, msg, '\n', 0 );
 	return( 0 );
