@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: lpd_jobs.c,v 3.0 1996/05/19 04:05:39 papowell Exp $";
+"$Id: lpd_jobs.c,v 3.3 1996/08/31 21:11:58 papowell Exp papowell $";
 
 #include "lpd.h"
 #include "printcap.h"
@@ -357,9 +357,11 @@ void Do_queue_jobs( char *name )
 	pid_t result = 0;			/* result pid and status */
 	plp_status_t procstatus;
 	int status;					/* status of the job */
-	int i;
+	int i, j;
 	char *pname, *s;				/* ACME, the very finest pointers */
 	int err;
+	struct destination *destination = 0;	/* current destination */
+	int can_remove;				/* can we remove this */
 
 	/* when a subserver, we come here to re-do the work
 		of reading the printcap file, etc. */
@@ -396,6 +398,17 @@ newprinter:
 		/* here we simply exit */
 		exit( JSUCC );
 	}
+	DEBUG4( "Do_queue_jobs: total_cf_files '%d'", C_files_list.count );
+	cfpp = (void *)C_files_list.list;
+	if(Debug>4){
+		for(i=0; i < C_files_list.count; ++i ){
+			cfp = cfpp[i];
+			logDebug("[%d] '%s'", i, cfp->name );
+			if( Debug>9 ){
+				dump_control_file( "Do_queue_jobs", cfp );
+			}
+		}
+	}
 
 	if( Printing_disabled ){
 		DEBUG0( "Do_queue_jobs: printing disabled" );
@@ -474,9 +487,12 @@ rescan:
 
 			/* see if you can print it */
 
+			destination = 0;
+			DEBUG4("Do_queue_jobs: checking '%s'",
+				cfp->name );
 			status = Check_printable( cfp );
-			DEBUG8("Do_queue_jobs: Check_printable '%s' = %d",
-				cfp->name, status );
+			DEBUG8("Do_queue_jobs: Check_printable '%s', status = %d (%s)",
+				cfp->name, status, Server_status(status) );
 			if( status ) goto done;
 
 			/*
@@ -490,6 +506,46 @@ rescan:
 				Fix_update( debug_vars, 0 );
 				goto rescan;
 			}
+			/* we may be processing an entry which is routed to a new
+				destination */
+
+next_destination:
+			if( cfp->routed_time ){
+				/* check to see if anything left */
+				can_remove = 1;
+				for( j = 0; j < cfp->destination_list.count; ++j ){
+					destination = (void *)cfp->destination_list.list;
+					destination = &destination[j];
+					if( destination->error[0] ){
+						can_remove = 0;
+						continue;
+					}
+					if( destination->copies > 0 &&
+							destination->copy_done >= destination->copies ){
+						destination->done = time( (void *) 0 );
+					}
+					if( destination->done ) continue;
+					break;
+				}
+				/* check to see if there is work to be done
+				 * if not, we do the next job in the list
+				 * Note that we may have a bounce queue destination
+				 */
+				if( j >= cfp->destination_list.count ){
+					DEBUG4(
+						"Do_queue_jobs: all destinations checked, remove %d",
+							can_remove );
+					destination = 0;
+					cfp->done_time = time( (void *)0 );
+					cfp->error[0] = 0;
+					if( can_remove ){
+						status = JREMOVE;
+					} else {
+						status = JIGNORE;
+					}
+					goto done;
+				}
+			}
 
 			/*
 			 * fork the process to do the actual dirty work
@@ -501,6 +557,7 @@ rescan:
 			 * The network database access stuff was especially nasty on
 			 *  some systems, i.e.- remote printer access
 			 */
+
 			if( (pid = dofork()) == 0 ){
 				/*
 				 * daughter process
@@ -512,15 +569,7 @@ rescan:
 				 * make sure that you can update the hold file
 				 */
 				cfp->active = pid;
-				if( Set_job_control( cfp ) ){
-					/* you cannot update hold file!! */
-					setstatus( NORMAL,
-						"cannot update hold file for '%s'", cfp->name );
-					log( LOG_ERR,
-						"Do_queue_jobs: cannot update hold file for '%s'", 
-						cfp->name );
-					exit( JREMOVE );
-				}
+				Destination = destination;
 				(void) plp_signal (SIGUSR1,(plp_sigfunc_t)cleanup_noerr );
 				(void) plp_signal (SIGHUP,(plp_sigfunc_t)cleanup_noerr );
 				(void) plp_signal (SIGINT,(plp_sigfunc_t)cleanup_err );
@@ -531,19 +580,60 @@ rescan:
 					s = cfp->redirect;
 					DEBUG4( "Do_queue_jobs: redirect %s to %s",
 						cfp->name, s );
+				} else if( destination ){
+					char copyname[16];
+					copyname[0] = 0;
+					destination->active = pid;
+					if( destination->copies ){
+						plp_snprintf( copyname,
+							sizeof(copyname),
+							"C%d", destination->copy_done+1 );
+					}
+					DEBUG4( "Do_queue_jobs: destination id before '%s', dest id '%s'",
+						cfp->identifier, destination->identifier );
+					if( cfp->IDENTIFIER ){
+						cfp->IDENTIFIER[0] = 0;
+						cfp->IDENTIFIER = 0;
+					}
+					plp_snprintf( cfp->ident_buffer,
+						sizeof( cfp->ident_buffer),
+						"A%s%s", destination->identifier, copyname );
+					DEBUG4( "Do_queue_jobs: ident_buffer '%s'",
+						cfp->ident_buffer );
+					cfp->IDENTIFIER = Prefix_job_line(cfp,cfp->ident_buffer);
+					cfp->identifier = cfp->IDENTIFIER+1;
+					s = destination->destination;
+					DEBUG4( "Do_queue_jobs: destination id after '%s', ID '%s'",
+						cfp->identifier, cfp->IDENTIFIER );
 				} else if( Bounce_queue_dest ){
 					s = Bounce_queue_dest;
 					DEBUG4( "Do_queue_jobs: bounce queue %s to %s",
 						cfp->name, s );
 				}
+				if( Set_job_control( cfp ) ){
+					/* you cannot update hold file!! */
+					setstatus( cfp,
+						"cannot update hold file for '%s'", cfp->name );
+					log( LOG_ERR,
+						"Do_queue_jobs: cannot update hold file for '%s'", 
+						cfp->name );
+					exit( JREMOVE );
+				}
 				if( s ){
 					DEBUG4( "Do_queue_jobs: sending %s job to %s",
 						cfp->name, s );
 					if( strchr( s, '@' ) == 0 ){
-						Errorcode = JABORT;
-						fatal( LOG_ERR,
-						"Do_queue_jobs: '%s' new destination '%s' missing host",
-							cfp->name, s );
+						/* append localhost if no host present */
+						if( strchr( s, '%' ) ){
+							Errorcode = JABORT;
+							fatal( LOG_ERR,
+							"Do_queue_jobs: '%s' new destination '%s' missing host",
+								cfp->name, s );
+						}
+						/* use localhost as the default destination */
+						plp_snprintf( error, sizeof(error), "%s@localhost",
+							s );
+						s = safestrdup( error );
 					}
 					RemoteHost = Orig_RemoteHost;
 					RemotePrinter = Orig_RemotePrinter;
@@ -589,51 +679,102 @@ rescan:
 				status = WEXITSTATUS( procstatus );
 			}
 
-			setstatus( NORMAL, "subserver status '%s' for '%s' on attempt %d",
-				Server_status(status), cfp->name, cfp->print_attempts+1 );
+			if( destination ){
+				char copyname[16];
+				copyname[0] = 0;
+				if( destination->copies ){
+					plp_snprintf( copyname,
+						sizeof(copyname),
+						"C%d", destination->copy_done+1 );
+				}
+				setstatus( cfp, "subserver status '%s' for '%s%s' destination '%s' on attempt %d",
+					Server_status(status), destination->identifier,copyname,
+					destination->destination,
+					destination->attempt+1 );
+			} else {
+				setstatus( cfp, "subserver status '%s' for '%s' on attempt %d",
+					Server_status(status), cfp->identifier,
+					cfp->print_attempts+1 );
+			}
 
 done:
 			DEBUG4( "Do_queue_jobs: final status %d, '%s'",
 				status, Server_status(status) );
 			switch( status ){
 			case JHOLD:
-				if( cfp->error[0] == 0 ){
+				if( destination ){
+					plp_snprintf( destination->error, sizeof(destination->error),
+						"error printing job, holding for retry" ); 
+					destination->hold = time( (void *) 0 );
+				} else {
 					plp_snprintf( cfp->error, sizeof(cfp->error),
-					"error printing job, holding for retry" ); 
+						"error printing job, holding for retry" ); 
+					cfp->hold_time = time( (void *)0 );
 				}
-				cfp->hold_time = time( (void *)0 );
 				Set_job_control( cfp );
+				++entry;
 				break;
 
 			case JIGNORE:	/* ignore this job for now */
+				if( destination ){
+					destination->ignore = 1;
+				}
 				break;
 
 			case JSUCC:	/* successful, remove job */
-				if( Remove_job( cfp ) ){
-					setstatus( NORMAL, "could not remove job %s",
-						cfp->name );
-					status = JABORT;
-					Errorcode = JABORT;
-					cleanup( 0 );
-				}; /**/
-				Sendmail_to_user( status, cfp, &pc_used );
+				if( destination ){
+					if( destination->copies == 0 ||
+						++destination->copy_done >= destination->copies ){
+						destination->done = time( (void *) 0 );
+					}
+					Set_job_control( cfp );
+				} else {
+					cfp->done_time = time( (void *)0 );
+					Set_job_control( cfp );
+					setmessage( cfp, "TRACE", "%s@%s: job printed", Printer, FQDNHost );
+					if( !Save_when_done && Remove_job( cfp ) ){
+						setstatus( cfp, "could not remove job %s",
+							cfp->name );
+						status = JABORT;
+						Errorcode = JABORT;
+						cleanup( 0 );
+					}
+					Sendmail_to_user( status, cfp, &pc_used );
+				}
 				++entry;
 				break;
 
 			default:
-				/* This is not good. You proably had a core dump */
-				setstatus( NORMAL, "subserver pid %d died!!! %s",
-					result, Decode_status( &procstatus ) );
+				/* This is not good. You probably had a core dump */
+				if( destination ){
+					plp_snprintf( destination->error,
+						sizeof(destination->error),
+						"subserver pid %d died!!! %s",
+						result, Decode_status( &procstatus ) );
+				} else {
+					plp_snprintf( cfp->error, sizeof(cfp->error),
+						"subserver pid %d died!!! %s",
+						result, Decode_status( &procstatus ) );
+					setstatus( cfp, "%s", cfp->error );
+				}
+				Set_job_control( cfp );
 				status = JABORT;
 				Errorcode = JABORT;
 				cleanup( 0 );
 				break;
 
-			case JFAIL:	/* failed, retry */
-				++cfp->print_attempts;
-				if( Send_try > 0
-					&& cfp->print_attempts >= Send_try ){
-					char *s;
+			case JFAIL:	/* failed, retry ?*/
+				j = 0;
+				if( destination ){
+					++destination->attempt;
+					j =  Send_try > 0
+						&& destination->attempt >= Send_try;
+				} else {
+					++cfp->print_attempts;
+					j =  Send_try > 0
+						&& cfp->print_attempts >= Send_try;
+				}
+				if( j ){
 					char buf[60];
 					/* check to see what the failure action
 					 *	should be - abort, failure; default is remove
@@ -651,34 +792,56 @@ done:
 						s = buf;
 						break;
 					}
-					setstatus( NORMAL, "too many failures on job '%s', %s",
-					cfp->name, s );
+					setstatus( cfp, "too many failures on job '%s', %s",
+						cfp->identifier, s );
 					Sendmail_to_user( status, cfp, &pc_used );
 					goto done;
-				} else {
-					--i;	/* retry the same job */
-					if( Connect_interval > 0 ){
-						setstatus( NORMAL, "sleeping %d secs before retry",
-							Connect_interval );
-						sleep( Connect_interval );
-					}
+				}
+				if( Connect_interval > 0 ){
+					setstatus( cfp, "sleeping %d secs before retry",
+						Connect_interval );
+					sleep( Connect_interval );
 				}
 				++entry;
 				break;
 
 			case JABORT:	/* abort, do not try again */
-				setstatus( NORMAL, "aborting operations on job %s",
-					cfp->name );
-				Sendmail_to_user( status, cfp, &pc_used );
+				if( destination ){
+					plp_snprintf( destination->error,
+						sizeof(destination->error),
+						"aborting operations on job %s, destination %s",
+							destination->identifier, destination->destination );
+				} else {
+					plp_snprintf( cfp->error,
+						sizeof(cfp->error),
+						"aborting operations on job %s",
+						cfp->identifier );
+					setstatus( cfp, "%s", cfp->error );
+					Sendmail_to_user( status, cfp, &pc_used );
+				}
+				Set_job_control( cfp );
 				Errorcode = JABORT;
 				cleanup( 0 );
 				break;
 
 			case JREMOVE:	/* failed, remove job */
+				if( destination ){
+					plp_snprintf( destination->error,
+						sizeof(destination->error),
+						"removing job entry due to failures" );
+					destination->done = time( (void *)0 );
+					Set_job_control( cfp );
+					break;
+				}
+				plp_snprintf( cfp->error,
+					sizeof(cfp->error),
+					"removing job due to failures" );
+				cfp->done_time = time( (void *)0 );
+				Set_job_control( cfp );
 				Sendmail_to_user( status, cfp, &pc_used );
 				if( Remove_job( cfp ) ){
-					setstatus( NORMAL, "could not remove job %s",
-						cfp->name );
+					setstatus( cfp, "could not remove job %s",
+						cfp->identifier );
 					status = JABORT;
 					Errorcode = JABORT;
 					cleanup( 0 );
@@ -686,6 +849,10 @@ done:
 				++entry;
 				break;
 			}
+			/* now we do the next destinations */
+			if( destination ){
+				goto next_destination;
+			}
 		}
 	} while( entry );
 	setstatus( NORMAL, "finished operations" );
@@ -716,15 +883,15 @@ int Remote_job( struct control_file *cfp, struct pc_used *pc_used )
 	case JIGNORE:
 		break;
 	case LINK_ACK_FAIL:
-		setstatus( NORMAL, "link failure while sending job '%s'", cfp->name );
+		setstatus( cfp, "link failure while sending job '%s'", cfp->identifier );
 		status = JABORT;
 		break;
 	case LINK_PERM_FAIL:
-		setstatus( NORMAL, "no permission to spool job '%s'", cfp->name );
+		setstatus( cfp, "no permission to spool job '%s'", cfp->identifier );
 		status = JREMOVE;
 		break;
 	default:
-		setstatus( NORMAL, "failed to send job '%s'", cfp->name );
+		setstatus( cfp, "failed to send job '%s'", cfp->identifier );
 		status = JFAIL;
 		break;
 	}
@@ -863,10 +1030,14 @@ rescan:
 			case JSUCC: /* printable */
 				break;
 			case JREMOVE:	/* remove job */
-				Sendmail_to_user( status, cfp, pc_used );
-				if( Remove_job( cfp ) ){
-					setstatus( NORMAL, "could not remove job %s",
-						cfp->name );
+				plp_snprintf( cfp->error, sizeof(cfp->error),
+					"job not printable, holding for retry" ); 
+				setstatus( cfp, "%s", cfp->error );
+				cfp->hold_time = time( (void *)0 );
+				(void)Set_job_control( cfp );
+				if( !Save_when_done && Remove_job( cfp ) ){
+					setstatus( cfp, "could not remove job %s",
+						cfp->identifier );
 					status = JABORT;
 					Errorcode = JABORT;
 					cleanup( 0 );
@@ -935,11 +1106,11 @@ rescan:
 				cfp->move_time = Server_pid;
 				if( Set_job_control( cfp ) ){
 					/* you cannot update hold file!! */
-					setstatus( NORMAL,
-						"cannot update hold file for '%s'", cfp->name );
+					setstatus( cfp,
+						"cannot update hold file for '%s'", cfp->identifier );
 					log( LOG_ERR,
 						"Start_subservers: cannot update hold file for '%s'", 
-						cfp->name );
+						cfp->identifier );
 					/* force it to be ignored and removed */
 					cfp->flags = 1;
 				} else {
@@ -952,9 +1123,9 @@ rescan:
 							add_buffer( &cfp->control_file, strlen(s)+1);
 						strcpy( cfp->spool_dir, s );
 						src_cfp = cfp;
-						setstatus( NORMAL,
+						setstatus( cfp,
 							"sending job '%s' to subserver '%s'",
-								cfp->name, server_info->name );
+								cfp->identifier, server_info->name );
 						return( server_info->name );
 					}
 				}
@@ -974,7 +1145,7 @@ rescan:
 			for( j = 0; j < servers.count && server_info[j].pid; ++j );
 			if( j < servers.count ){
 				DEBUG4("Start_subservers: all completed, none available" );
-				setstatus( NORMAL,"no subserver available to print jobs" );
+				setstatus( cfp,"no subserver available to print jobs" );
 				exit(0);
 			} else {
 				sleep(10);
@@ -1246,7 +1417,7 @@ static void Copy_job()
 	Errorcode = JABORT;
 
 	DEBUG7("Copy_job: printer '%s', job '%s' copying data files",
-		Printer, src_cfp->name );
+		Printer, src_cfp->identifier );
 		
 	Init_path( &srcdpath, src_cfp->spool_dir );
 	data_file = (void *)src_cfp->data_file.list;
@@ -1258,7 +1429,7 @@ static void Copy_job()
 		/* try renaming */
 		link_or_copy( src, dest, 0 );
 	}
-	s = src_cfp->name;
+	s = src_cfp->identifier;
 	src = Add_path( &srcdpath, s );
 	dest = Add_path( SDpathname, s );
 	DEBUG7("Copy_job: control file src '%s' dest '%s'", src, dest );
@@ -1273,7 +1444,7 @@ static void Copy_job()
 			logerr( LOG_ERR, "Copy_job: cannot unlink '%s'", src );
 		}
 	}
-	s = src_cfp->name;
+	s = src_cfp->identifier;
 	src = Add_path( &srcdpath, s );
 	DEBUG7("Copy_job: unlink src '%s'", src );
 	if( stat( src, &statb ) == 0 && unlink( src ) == -1 ){
@@ -1336,6 +1507,7 @@ static int Check_printable( struct control_file *cfp )
 		|| (cfp->move_time == Server_pid)
 		|| cfp->hold_time
 		|| cfp->remove_time
+		|| cfp->done_time
 		){
 		return(JIGNORE);
 	}
@@ -1346,11 +1518,11 @@ static int Check_printable( struct control_file *cfp )
 	cfp->active = 0;
 	if( Set_job_control( cfp ) ){
 		/* you cannot update hold file!! */
-		setstatus( NORMAL,
-			"cannot update hold file for '%s'", cfp->name );
+		setstatus( cfp,
+			"cannot update hold file for '%s'", cfp->identifier );
 		log( LOG_ERR,
 			"Check_printable: cannot update hold file for '%s'", 
-			cfp->name );
+			cfp->identifier );
 		/* force it to be ignored and removed */
 		cfp->flags = 1;
 		return( JREMOVE );
@@ -1377,8 +1549,8 @@ static int Check_printable( struct control_file *cfp )
 	if( Perms_check( &Perm_file, &Perm_check, cfp ) == REJECT
 		|| Perms_check( &Local_perm_file, &Perm_check, cfp ) == REJECT
 		|| Last_default_perm == REJECT ){
-		setstatus( NORMAL,
-			"no permission to print job %s", cfp->name );
+		setstatus( cfp,
+			"no permission to print job %s", cfp->identifier );
 		return( JREMOVE );
 	}
 	return( JSUCC );
