@@ -11,13 +11,14 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: lpd_rcvjob.c,v 3.3 1996/07/11 02:55:21 papowell Exp papowell $";
+"$Id: lpd_rcvjob.c,v 3.7 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lpd.h"
 #include "printcap.h"
 #include "lp_config.h"
 #include "permission.h"
 #include "freespace.h"
+#include "jobcontrol.h"
 
 /***************************************************************************
 Commentary:
@@ -91,6 +92,7 @@ static struct control_file cf_static;	/* control file for job */
 
 static char *tempname;			/* temporary control file */
 static char *controlname;		/* real control file */
+static char *holdfilename;		/* hold file */
 
 static struct malloc_list data_files; /* list of data files received */
 
@@ -116,6 +118,10 @@ static void Remove_files( void *p )
 		DEBUG4("Remove_files: unlinking %s", tempname );
 		unlink( tempname );
 	}
+	if( holdfilename ){
+		DEBUG4("Remove_files: unlinking %s", holdfilename );
+		unlink( holdfilename );
+	}
 }
 
 
@@ -146,6 +152,7 @@ int Receive_job( int *socket, char *input, int maxlen )
 	char *orig_name;			/* original printer name */
 	int max_job_number;			/* maximum job number */
 	char **lines;				/* lines in the file */
+	char cfile_copy[LARGEBUFFER];	/* control file copy */
 
 	/* clear error message */
 	error[0] = 0;
@@ -204,7 +211,7 @@ int Receive_job( int *socket, char *input, int maxlen )
 	do{
 		DEBUG3("Receive_job: from %s- getting file transfer line", FQDNRemote );
 		len = sizeof(line)-1;
-		line[0] = 0;
+		line[0] = 0; line[1] = 0;
 		status = Link_line_read( ShortRemote, socket, Send_timeout,
 			line, &len );
 
@@ -252,7 +259,7 @@ int Receive_job( int *socket, char *input, int maxlen )
 		openname = safestrdup( filename );	/* name we get from remote */
 		if( Check_format( line[0], openname, &cf_static ) ){
 			plp_snprintf( error, sizeof(error),
-				"%s: file '%' name format not [cd]f[A-Za-z]NNNhost",
+				"%s: file '%s' name format not [cd]f[A-Za-z]NNNhost",
 					Printer, filename );
 			ack = ACK_FAIL;
 			goto error;
@@ -602,7 +609,7 @@ int Receive_job( int *socket, char *input, int maxlen )
 			 */
 			s = jobfiles[i].openname;
 			DEBUG8("Receive_job: control file line %d data file '%s' = '%s'",
-				jobfiles[i].line, s, lines[jobfiles[i].line] /**/ );
+				jobfiles[i].line, s, lines[jobfiles[i].line]+1 /**/ );
 			missing = 1;
 			for( j = 0;
 				j < data_files.count
@@ -610,11 +617,10 @@ int Receive_job( int *socket, char *input, int maxlen )
 			if( missing == 0 ){
 				/* back up one place on the transfername and put right
 					value into control file */
-				--datafile[j].transfername;
-				datafile[j].transfername[0] = lines[jobfiles[i].line][0];
-				lines[jobfiles[i].line] = datafile[j].transfername;
+				datafile[j].transfername[-1] = lines[jobfiles[i].line][0];
+				lines[jobfiles[i].line] = datafile[j].transfername-1;
 				if( jobfiles[i].Uinfo ){
-					s = safestrdup( datafile[j].transfername );
+					s = safestrdup( datafile[j].transfername-1 );
 					s[0] = 'U';
 					lines[jobfiles[i].uline] = s;
 				}
@@ -649,6 +655,8 @@ int Receive_job( int *socket, char *input, int maxlen )
 		/*
 		 * truncate the control file and rewrite it
 		 */
+		Remove_job_control( cf );
+		holdfilename = cf->hold_file;
 		if( ftruncate( temp_fd, 0 ) ){
 			err = errno;
 			plp_snprintf( error, sizeof( error),
@@ -658,26 +666,40 @@ int Receive_job( int *socket, char *input, int maxlen )
 		}
 		if( Use_queuename && cf->QUEUENAME == 0 ){
 			char qn[M_QUEUENAME];
-			plp_snprintf( qn, sizeof(qn), "Q%s\n", orig_name );
+			plp_snprintf( qn, sizeof(qn), "Q%s", orig_name );
 			DEBUG4("Receive_job: printer '%s' adding '%s'", orig_name, qn );
-			if( Write_fd_str( temp_fd, qn ) < 0 ){
-				err = errno;
-				plp_snprintf( error, sizeof( error),
-					"write of '%s' failed - %s", tempname, Errormsg(err));
-				ack = ACK_RETRY;
-				goto error;
-			}
+			cf->QUEUENAME = Prefix_job_line( cf, qn );
 		}
+		if( (Routing_filter || Use_identifier) && cf->IDENTIFIER == 0 ){
+			DEBUG4("Receive_job: printer '%s' adding '%s'",
+				orig_name, cf->identifier );
+			cf->IDENTIFIER = Prefix_job_line( cf, cf->identifier-1 );
+		}
+		/*
+		 * Now we do the routing
+		 */
+		DEBUG4("Receive_job: Routing_filter '%s'", Routing_filter );
+		if( Routing_filter && *Routing_filter && Get_route( cf ) ){
+			plp_snprintf( error, sizeof( error),
+				"cannot do routing for %s", orig_name );
+			ack = ACK_RETRY;
+			goto error;
+		}
+
+
 		/*
 		 * now we fix up the control file, and write it out again
 		 * if necessary
 		 */
 		lines = (char **)cf->info_lines.list;
 		DEBUG4("Receive_job: info_lines.count '%d'", cf->info_lines.count );
+		cfile_copy[0] = 0;
 		for(i = 0; i < cf->info_lines.count; ++i ){
 			/* we get the pointer to the location in file */
-			DEBUG4("Receive_job: line[%d]='%s'", i, lines[i] );
-			if( Write_fd_str( temp_fd, lines[i] ) < 0
+			s = lines[i];
+			DEBUG4("Receive_job: line[%d]='%s'", i, s );
+			if( s == 0 || *s == 0 ) continue;
+			if( Write_fd_str( temp_fd, s ) < 0
 				|| Write_fd_str( temp_fd, "\n" ) < 0  ){
 				err = errno;
 				plp_snprintf( error, sizeof( error),
@@ -685,6 +707,15 @@ int Receive_job( int *socket, char *input, int maxlen )
 				ack = ACK_RETRY;
 				goto error;
 			}
+			if( Logger_destination ){
+				safestrncat( cfile_copy, s );
+				safestrncat( cfile_copy, "\n" );
+			}
+		}
+		if( Set_job_control( cf ) ){
+			plp_snprintf( error, sizeof( error), "could not write hold file" );
+			ack = ACK_RETRY;
+			goto error;
 		}
 
 		if( temp_fd >= 0 ){
@@ -715,6 +746,8 @@ done:
 		} else {
 			s = Printer;
 		}
+		setmessage( cf, "TRACE", "%s@%s: job arrived\n%s",
+			Printer, FQDNHost, cfile_copy );
 		plp_snprintf( line, sizeof(line), "!%s\n", s );
 		DEBUG4("Receive_job: sending '%s' to LPD", s );
 		if( Write_fd_str( Lpd_pipe[1], line ) < 0 ){
@@ -726,8 +759,7 @@ done:
 	return(status);
 
 error:
-	status = 1;
-	Remove_files( 0 );
+	status = JREMOVE;
 	log( LOG_INFO, "Receive_job: error '%s'", error );
 	if( error[0] ){
 		DEBUG3("Receive_job: sending ACK %d, msg '%s'", ack, error );
