@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: lpd_status.c,v 3.0 1996/05/19 04:05:43 papowell Exp $";
+"$Id: lpd_status.c,v 3.4 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lpd.h"
 #include "printcap.h"
@@ -19,6 +19,7 @@ static char *const _id =
 #include "decodestatus.h"
 #include "jobcontrol.h"
 #include "permission.h"
+#include "globmatch.h"
 
 /***************************************************************************
 Commentary:
@@ -55,14 +56,16 @@ Format of the information reporting:
  Rank  Owner           Class Job  Files                                 Size
 1      papowell@hostname   A 040  standard input                        3
 2      papowell@hostname   A 041  standard input                        3
+                                                                        ^ MAXLEN
 
  ***************************************************************************/
 
+#define MAXLEN 69
 void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 	int tokencount, struct token *tokens );
 void Get_queue_server( char *name, int *socket, int longformat, int allflag,
 	int tokencount, struct token *tokens );
-
+void Print_status_info( int *socket, int longformat, char *path, char *header );
 
 int Job_status( int *socket, char *input, int maxlen )
 {
@@ -224,7 +227,6 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 	char msg[SMALLBUFFER];
 	char *s, *host, *logname; /* ACME pointer for writing */
 	int i, j, status, len, count, hold_count;
-	char line[LINEBUFFER];
 	char number[LINEBUFFER];		/* printable form of small integer */
 	int serverpid, unspoolerpid;	/* pid of server and unspooler */
 	int fd;					/* file descriptor for file */
@@ -235,7 +237,9 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 	int select;				/* select this job for display */
 	char *path;				/* path to use */
 	char *pr;
-	char *endbuffer;		/* end of the buffer */
+	int jobnumber;			/* job number */
+	struct destination *destination, *d;
+	int nodest;				/* no destination information */
 
 	/* set printer name and printcap variables */
 	DEBUG4( "Get_queue_status: checking '%s'", name );
@@ -352,8 +356,8 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 	if( Bounce_queue_dest ){
 		len = strlen( msg );
 		plp_snprintf( msg+len, sizeof(msg) - len,
-			" (bounce queue to '%s')",
-			Bounce_queue_dest );
+			" (%sbounce queue to '%s')",
+			Routing_filter?"routed/":"", Bounce_queue_dest );
 	}
 
 	/*
@@ -406,11 +410,14 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 
 
 	/* set up the short format for folks */
+	cfpp = (void *)C_files_list.list;
+	job_count( &hold_count, &count );
+
 	/* this gives a short 1 line format with minimum info */
 	if( !longformat ){
 		len = strlen( msg );
 		plp_snprintf( msg+len, sizeof(msg) - len, " %d jobs",
-			C_files_list.count );
+			count );
 	}
 
 	if( Link_send( ShortRemote, socket, Send_timeout,
@@ -444,21 +451,6 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 		unspoolerpid = 0;
 	} /**/
 
-	cfpp = (void *)C_files_list.list;
-	count = 0;
-	hold_count = 0;
-	for( i = 0; i < C_files_list.count; ++i ){
-		cfp = cfpp[i];
-		if( cfp->error[0] || cfp->remove_time || cfp->move_time
-			|| cfp->redirect[0] || cfp->held_class ){
-			continue;
-		}
-		if( cfp->hold_time ){
-			++hold_count;
-			continue;
-		}
-		++count;
-	}
 	if( count == 0 ){
 		if( Link_send( ShortRemote, socket, Send_timeout,
 			0x00, "  Queue: no printable jobs in queue", '\n', 0 ) ) exit(0);
@@ -501,126 +493,59 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 	/*
 	 * get the last status of the spooler
 	 */
+	DEBUG8("Get_queue_status: Max_status_size '%d'", Max_status_size );
 	if( buffer == 0 ){
-		DEBUG8("Get_queue_status: Status '%s'", buffer );
 		bsize = Max_status_size*1024;
 		if( bsize == 0 ) bsize = 1024;
 		malloc_or_die( buffer, bsize+2 );
 	}
 	path = Add2_path( CDpathname, "status.", Printer );
+	Print_status_info( socket, longformat, path, "Status" );
 
-	if( (fd = Checkread( path, &statb ) ) >= 0 ){
-		off_t off = 0;
-		if( statb.st_size > bsize ) off = statb.st_size - bsize;
-        if( lseek( fd, off, SEEK_SET ) < 0 ){
-            logerr_die( LOG_ERR, "setstatus: cannot seek '%s'", path );
-        }
-		for( len = bsize, s = buffer;
-			len > 0 && (i = read( fd, s, len ) ) > 0;
-			len -= i, s += i );
-		*s = 0;
-		endbuffer = s;
-		/*
-		 * now we back up and print out only the last 5 lines of status
-		 */
-		for( i = 0;
-			(longformat || i < 5) && (s = strrchr( buffer, '\n' )); ){
-			if( strlen( s+1 ) ) ++i;
-			*s++ = 0;
-		}
-		if( s == 0 ){
-			s = buffer;
-		}
-		while( s < endbuffer ){
-			if( *s ){
-				plp_snprintf( line, sizeof(line), "  Status: %s", s );
-				if( Link_send( ShortRemote, socket, Send_timeout,
-					0x00, line, '\n', 0 ) ) exit(0);
-			}
-			s += strlen(s)+1;
-		}
-	}
-	close(fd);
-
-
-	/*
-	 * get the last status of the filter
-	 */
 	if( Status_file && *Status_file ){
 		if( Status_file[0] == '/' ){
 			path = Status_file;
 		} else {
 			path = Add_path( SDpathname, Status_file );
 		}
-		if( (fd = Checkread( path, &statb ) ) >= 0 ){
-			off_t off = 0;
-			if( statb.st_size > bsize ) off = statb.st_size - bsize;
-			if( lseek( fd, off, SEEK_SET ) < 0 ){
-				logerr_die( LOG_ERR, "setstatus: cannot seek '%s'", path );
-			}
-			for( len = bsize, s = buffer;
-				len > 0 && (i = read( fd, s, len ) ) > 0;
-				len -= i, s += i );
-			*s = 0;
-			endbuffer = s;
-			/*
-			 * now we back up and print out only the last 5 lines of status
-			 */
-			s = 0;
-			for( i = 0;
-				(longformat || i < 5) && (s = strrchr( buffer, '\n' )); ){
-				if( strlen(s+1) ) ++i;
-				*s++ = 0;
-			}
-			if( s == 0 ) s = buffer;
-			while( s < endbuffer ){
-				if( *s ){
-					plp_snprintf( line,sizeof(line),"  Filter_status: %s", s );
-					if( Link_send( ShortRemote, socket, Send_timeout,
-						0x00, line, '\n', 0 ) ) exit(0);
-				}
-				s += strlen(s)+1;
-			}
-		}
-		close(fd);
+		Print_status_info( socket, longformat, path, "Filter_status" );
 	}
 
 	if( C_files_list.count > 0 ){
+		/* send header */
+		s =
+" Rank  Owner/ID        Class Job Files                           Size Time";
 		if( Link_send( ShortRemote, socket, Send_timeout,
-			0x00, 
-/*
-0        1         2         3         4         5         6         7
-12345678901234567890123456789012345678901234567890123456789012345678901234 */
-" Rank  Owner           Class Job Files                           Size Time",
-			'\n', 0 ) ) exit( 0 );
+			0x00,  s, '\n', 0 ) ) exit( 0 );
+		jobnumber = 0;
 		for( i = 0; i < C_files_list.count; ++i ){
 			cfp = cfpp[i];
 
+			DEBUG8("Get_queue_status: job name '%s' id '%s'",
+				cfp->name, cfp->identifier );
+
 			/*
 			 * check to see if this entry matches any of the patterns
 			 */
 
-			select = 1;
+			destination = 0;
 			if( tokencount > 0 ){
+				select = 1;
 				for( j = 0; select && j < tokencount; ++j ){
-					select = patselect( &tokens[j], cfp );
+					select = patselect( &tokens[j], cfp, &destination );
 				}
+				DEBUG8("Get_queue_status: job name '%s' select '%d'",
+					select );
+				if( !select ) continue;
 			}
-			if( !select ) continue;
+			/* set a flag to suppres destinations */
+			nodest = 0;
 
 			/* we report this jobs status */
 
 			error[0] = 0;
 			if( cfp->JOBNAME ){
-				s = cfp->JOBNAME + 1;
-				safestrncat( error, s );
-				/* watch out for this! */
-				error[32] = 0;
-			}
-			if( cfp->error[0] ){
-				error[0] = 0;
-				plp_snprintf( error, sizeof( error ),
-					"ERROR: %s", cfp->error );
+				safestrncat( error, cfp->JOBNAME+1 );
 			}
 			s = strrchr( cfp->name, '/' );
 			if( s ){
@@ -631,45 +556,121 @@ void Get_queue_status( char *name, int *socket, int longformat, int allflag,
 			priority = s[2];
 			host = cfp->FROMHOST?cfp->FROMHOST+1:"???";
 			if( (s = strchr( host, '.' )) ) *s = 0;
-			logname = cfp->LOGNAME?cfp->LOGNAME+1:"???",
-			plp_snprintf(line, sizeof(line), "%s@%s",
-				logname, host );
+			logname = cfp->LOGNAME?cfp->LOGNAME+1:"???";
 			number[0] = 0;
-			if( cfp->active ){
-				strcpy( number, "active" );
-			} else if( cfp->error[0] ){
+			if( cfp->error[0] ){
 				strcpy( number, "error" );
+				plp_snprintf( error, sizeof( error ),
+					"ERROR: %s", cfp->error );
+				nodest = 1;
 			} else if( cfp->hold_time ){
 				strcpy( number, "hold" );
+				nodest = 1;
 			} else if( cfp->move_time ){
 				strcpy( number, "move" );
+				nodest = 1;
 			} else if( cfp->remove_time ){
 				strcpy( number, "remove" );
+				nodest = 1;
+			} else if( cfp->done_time ){
+				strcpy( number, "done" );
+				nodest = -1;
 			} else if( cfp->held_class ){
 				strcpy( number, "class" );
-			} else {
-				plp_snprintf( number, sizeof(number), "%d", i+1 );
-			}
-			if( cfp->redirect[0] ){
-				j = strlen( number );
-				strncat( number+j, "->", sizeof(number));
-				j = strlen( number );
-				strncpy( number+j, cfp->redirect, sizeof(number)-j );
+			} else if( cfp->active ){
+				strcpy( number, "active" );
+				nodest = -1;
+			} else if( cfp->redirect[0] ){
+				plp_snprintf( number, sizeof(number), "redirect->%s",
+					cfp->redirect );
+				nodest = 1;
 			}
+
+			DEBUG8("Get_queue_status: destination count '%d', nodest %d",
+				cfp->destination_list.count, nodest );
 			/* do the number, owner, and job information */
-			plp_snprintf( msg, sizeof(msg), "%-6s %-19s %c %03d %-32s",
-				number, line, priority, cfp->number, error );
+			if( number[0] == 0 ){
+				plp_snprintf( number, sizeof(number), "%d",
+					jobnumber+1, cfp->redirect );
+			}
+			++jobnumber;
+			plp_snprintf( msg, sizeof(msg), "%-6s %-19s %c %03d %-s",
+				number, cfp->identifier, priority, cfp->number, error );
 			if( cfp->error[0] == 0 ){
+				char sizestr[16];
 				len = strlen(msg);
-				plp_snprintf( msg + len, sizeof(msg)-len, "%4d",
-					cfp->jobsize );
-				/* get the modification time */
-				len = strlen(msg);
-				plp_snprintf( msg+len, sizeof(msg)-len, " %s",
-					Time_str( 1, cfp->statb.st_ctime ) );
+				if( len > MAXLEN){
+					len = MAXLEN;
+				} else {
+					while( len < MAXLEN ){
+						msg[len++] = ' ';
+					}
+				}
+				plp_snprintf( sizestr, sizeof(sizestr), "%0d",cfp->jobsize);
+				len -= ( strlen( sizestr ) + 1 );
+				DEBUG8("Get_queue_status: MAXLEN '%d', sizestr '%s'",
+					len, sizestr );
+				plp_snprintf( msg+len, sizeof(msg)-len, " %s %s",
+					sizestr, Time_str( 1, cfp->statb.st_ctime ) );
 			}
 			if( Link_send( ShortRemote, socket, Send_timeout,
 				0x00, msg, '\n', 0 ) ) exit( 0 );
+			if( cfp->destination_list.count > 0 ){
+				/* put in the destination information */
+				destination = (void *)cfp->destination_list.list;
+				for( j = 0; j < cfp->destination_list.count; ++j ){
+					d = &destination[j];
+					error[0] = 0;
+					plp_snprintf( error, sizeof(error), "->%s ",
+						d->destination );
+					DEBUG8("Get_queue_status: destination active '%d'",
+						d->active );
+					if( d->active && kill( d->active, 0 ) ){
+						d->active = 0;
+					}
+					if( d->active ) ++d->copy_done;
+					if( d->copies > 1 ){
+						len = strlen( error );
+						plp_snprintf( error+len, sizeof(error)-len,
+							"<cpy %d/%d> ",
+							d->copy_done, d->copies );
+					}
+					safestrncpy( number, " -" );
+					if( d->error[0] ){
+						safestrncat( number, "rterror" );
+						len = strlen( error );
+						plp_snprintf( error+len, sizeof( error )-len,
+							"ERROR: %s", d->error );
+					} else if( d->active ){
+						safestrncat( number, "actv" );
+					} else if( d->hold ){
+						safestrncat( number, "hold" );
+					} else if( d->done ){
+						safestrncat( number, "done" );
+					}
+					plp_snprintf( msg, sizeof(msg), "%-6s %-19s %c %03d %-s",
+						number, d->identifier, priority, cfp->number, error );
+					if( d->error[0] == 0 ){
+						char sizestr[16];
+						len = strlen(msg);
+						if( len > MAXLEN){
+							len = MAXLEN;
+						} else {
+							while( len < MAXLEN ){
+								msg[len++] = ' ';
+							}
+						}
+						plp_snprintf( sizestr, sizeof(sizestr), "%0d",cfp->jobsize);
+						len -= ( strlen( sizestr ) + 1 );
+						DEBUG8("Get_queue_status: MAXLEN '%d', sizestr '%s'",
+							len, sizestr );
+						plp_snprintf( msg+len, sizeof(msg)-len, " %s %s",
+							sizestr, Time_str( 1, cfp->statb.st_ctime ) );
+					}
+					if( Link_send( ShortRemote, socket, Send_timeout,
+						0x00, msg, '\n', 0 ) ) exit( 0 );
+				}
+			}
 		}
 	}
 
@@ -746,13 +747,16 @@ error:
  *    in the control file:
  *  token is INTEGER: then matches the job number
  *  token is string: then matches either the user name or host name
+ *    then try glob matching job ID
  *
  ***************************************************************************/
 
-int patselect( struct token *token, struct control_file *cfp )
+int patselect( struct token *token, struct control_file *cfp,
+	struct destination **destination )
 {
 	char *s, *end;
-	int val, len;
+	int val, len, j;
+	struct destination *destination_list, *d;
 
 	s = token->start;
 	len = token->length;
@@ -760,25 +764,117 @@ int patselect( struct token *token, struct control_file *cfp )
 	DEBUG6("patselect: '%s'", s );
 
 	/* handle wildcard match */
-	if( strcmp( s, "all" ) == 0 ){
+	if( strcasecmp( s, "all" ) == 0 ){
 		return( 1 );
 	}
 	end = s;
 	val = strtol( s, &end, 10 );
-	if( end - s ){
+	if( (end - s) == len ){
 		/* we check job number */
 		DEBUG6("patselect: job number check '%d' to job %d",
 			val, cfp->number );
 		return( val == cfp->number );
-	}
-	/* now we check to see if we have a name match */
-	if( cfp->LOGNAME && strncasecmp( s, cfp->LOGNAME+1, len ) == 0 ){
-		DEBUG6("patselect: job logname '%s' match", cfp->LOGNAME );
-		return(1);
-	}
-	if( cfp->FROMHOST && strncasecmp( s, cfp->FROMHOST+1, len ) == 0 ){
-		DEBUG6("patselect: job host '%s' match", cfp->FROMHOST );
-		return(1);
+	} else {
+		/* now we check to see if we have a name match */
+		if( cfp->LOGNAME && strcasecmp( s, cfp->LOGNAME+1 ) == 0 ){
+			DEBUG6("patselect: job logname '%s' match", cfp->LOGNAME );
+			return(1);
+		}
+		if( strcasecmp( s, cfp->identifier ) == 0 ){
+			DEBUG6("patselect: job identifier '%s' match", cfp->identifier );
+			return(1);
+		}
+		if( destination && (j = cfp->destination_list.count) > 0 ){
+			destination_list = (void *)cfp->destination_list.list;
+			if( *destination ){
+				/* get next one */
+				d = *destination;
+				++d;
+			} else {
+				d = destination_list;
+			}
+			*destination = 0;
+			for( ;d < &destination_list[j]; ++d ){
+				if( strcasecmp( s, d->identifier ) == 0 ){
+					DEBUG6("patselect: job identifier '%s' match", d->identifier );
+					*destination = d;
+					return(1);
+				} else if( strchr( s, '*' ) && Globmatch( s, d->identifier) == 0 ){
+					DEBUG6("patselect: job identifier '%s' globmatch",d->identifier);
+					*destination = d;
+					return(1);
+				}
+			}
+		}
+		if( strchr( s, '*' ) && Globmatch( s, cfp->identifier) == 0 ){
+			DEBUG6("patselect: job identifier '%s' globmatch",cfp->identifier);
+			return(1);
+		}
 	}
 	return(0);
 }
+
+void Print_status_info( int *socket, int longformat, char *path, char *header )
+{
+	int fd, len, i;
+	off_t off = 0;
+	char *s, *endbuffer;
+	struct stat statb;		/* stat of file */
+	char line[LINEBUFFER];
+
+	if( (fd = Checkread( path, &statb ) ) >= 0 ){
+		if( statb.st_size > bsize ){
+			off = statb.st_size - bsize;
+			if( lseek( fd, off, SEEK_SET ) < 0 ){
+				logerr_die( LOG_ERR, "setstatus: cannot seek '%s'", path );
+			}
+		}
+		for( len = bsize, s = buffer;
+			len > 0 && (i = read( fd, s, len ) ) > 0;
+			len -= i, s += i );
+		*s = 0;
+		close(fd);
+		if( off && (s = strchr(buffer, '\n'))
+			&& (endbuffer = strrchr( s, '\n' )) ){
+			++s;
+			*++endbuffer = 0;
+		} else {
+			s = buffer;
+		}
+		/*
+		 * print out only the last complete lines
+		 */
+		while( s && *s ){
+			endbuffer = strchr( s, '\n' );
+			*endbuffer++ = 0;
+			plp_snprintf( line, sizeof(line), "  %s: %s", header, s );
+			if( Link_send( ShortRemote, socket, Send_timeout,
+				0x00, line, '\n', 0 ) ) exit(0);
+			s = endbuffer;
+		}
+	}
+}
+
+void job_count( int *hc, int *cnt )
+{
+	struct control_file *cfp, **cfpp;	/* pointer to control file */
+	int i, count, hold_count;
+
+	count = 0; hold_count = 0;
+	cfpp = (void *)C_files_list.list;
+	for( i = 0; i < C_files_list.count; ++i ){
+		cfp = cfpp[i];
+		if( cfp->error[0] || cfp->remove_time || cfp->move_time
+			|| cfp->redirect[0] || cfp->held_class
+			|| cfp->done_time ){
+			continue;
+		}
+		if( cfp->hold_time ){
+			++hold_count;
+			continue;
+		}
+		++count;
+	}
+	if( hc ) *hc = hold_count;
+	if( cnt ) *cnt = count;
+}
