@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: getcnfginfo.c,v 3.5 1996/07/20 02:42:11 papowell Exp papowell $";
+"$Id: getcnfginfo.c,v 3.7 1996/08/25 22:20:05 papowell Exp papowell $";
 
 #include "lp.h"
 #include "lp_config.h"
@@ -632,6 +632,8 @@ struct keywords lpd_config[]
  { "connect_timeout", INTEGER_K, &Connect_timeout },
  { "default_banner_printer", STRING_K, &Default_banner_printer },
  { "default_format", STRING_K, &Default_format },
+ { "default_logger_port", STRING_K, &Default_logger_port },
+ { "default_logger_protocol", STRING_K, &Default_logger_protocol },
  { "default_permission", STRING_K, &Default_permission },
  { "default_printer", STRING_K, &Default_printer },
  { "default_priority", STRING_K, &Default_priority },
@@ -643,6 +645,7 @@ struct keywords lpd_config[]
  { "group", STRING_K, &Server_group },
  { "lockfile", STRING_K, &Lockfile },
  { "logfile", STRING_K, &Logfile },
+ { "logger_destination", STRING_K, &Logger_destination },
  { "longnumber", INTEGER_K, &Long_number },
  { "lpd_port", STRING_K, &Lpd_port },
  { "lpd_printcap_path", STRING_K, &Lpd_printcap_path },
@@ -654,6 +657,8 @@ struct keywords lpd_config[]
  { "originate_port", STRING_K, &Originate_port },
  { "printcap_path", STRING_K, &Printcap_path },
  { "printer_perms_path", STRING_K, &Printer_perms_path },
+ { "router", STRING_K, &Routing_filter },
+ { "save_when_done", INTEGER_K, &Save_when_done },
  { "send_data_first", INTEGER_K, &Send_data_first },
  { "send_failure_action", STRING_K, &Send_failure_action },
  { "send_timeout", INTEGER_K, &Send_timeout },
@@ -663,6 +668,7 @@ struct keywords lpd_config[]
  { "spool_dir_perms", INTEGER_K, &Spool_dir_perms },
  { "spool_file_perms", INTEGER_K, &Spool_file_perms },
  { "syslog_device", STRING_K, &Syslog_device },
+ { "use_identifier", INTEGER_K, &Use_identifier },
  { "use_info_cache", INTEGER_K, &Use_info_cache },
  { "use_queuename", INTEGER_K, &Use_queuename },
  { "use_shorthost", INTEGER_K, &Use_shorthost },
