@@ -53,7 +53,7 @@
  ***************************************************************************/
 
 static char *const _id =
-"$Id: getopt.c,v 3.0 1996/05/19 04:06:00 papowell Exp $";
+"$Id: getopt.c,v 3.2 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lp.h"
 
@@ -62,7 +62,6 @@ int Opterr = 1;                 /* Zero disables errors msgs */
 char *Optarg;               /* Pointer to option argument */
 char *next_opt;			    /* pointer to next option char */
 char *Name;						/* Name of program */
-extern int Debug;				/* debug */
 char **Argv_p;
 int Argc_p;
 
@@ -92,7 +91,12 @@ Getopt (int argc, char *argv[], char *optstring)
 		if( basi ){
 			basename = basi + 1;
 		}
-		Name = (char *) safestrdup (basename);
+		Name = malloc( strlen(basename) + 1);
+		if( Name == 0 ){
+			fprintf( stderr, "Getopt: malloc failed\n");
+			exit(1);
+		}
+		strcpy(Name, basename);
 		Argv_p = argv;
 		Argc_p = argc;
 		Optind = 1;
@@ -164,7 +168,5 @@ Getopt (int argc, char *argv[], char *optstring)
 		}
 		next_opt = 0;
 	}
-	DEBUG9( "Getopt: return %c, Optind %d, Optarg '%s'",
-		option, Optind, Optarg?Optarg:"<NULL>" );
 	return (option);
 }
