@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: getqueue.c,v 3.1 1996/06/30 17:12:44 papowell Exp $";
+"$Id: getqueue.c,v 3.4 1996/08/31 21:11:58 papowell Exp papowell $";
 
 /***************************************************************************
 Commentary
@@ -329,6 +329,7 @@ int Parse_cf( struct dpathname *dpath, struct control_file *cf, int check_df )
 			df = (void *)cf->data_file.list;
 			df = &df[cf->data_file.count++];
 			/* get the format and name of file */
+			memset( df, 0, sizeof( df[0] ) );
 			df->format = s[0];
 			df->openname = s+1;
 			df->line = i;
@@ -354,6 +355,10 @@ int Parse_cf( struct dpathname *dpath, struct control_file *cf, int check_df )
 		df->linecount = i - df->line;
 	}
 
+	if( Make_identifier( cf ) ){
+		goto error;
+	}
+
 	DEBUG8("Parse_cf: before dump job number '%d', filehostname '%s'",
 		cf->number, cf->filehostname );
 
@@ -367,6 +372,52 @@ error:
 	return( 1 );
 }
 
+/************************************************************************
+ * Make_identifier - add an identifier field to the job
+ *  the identifier has the format name@host%id
+ ************************************************************************/
+int Make_identifier( struct control_file *cf )
+{
+	char *user = "nobody";
+	char *host = "unknown";
+	int len;
+
+	if( cf->IDENTIFIER ){
+		if( cf->IDENTIFIER[1] == 0 ){
+			plp_snprintf( cf->error, sizeof( cf->error),
+				"bad IDENTIFIER line in '%s'", cf->name );
+			return( 1 );
+		}
+		cf->identifier = cf->IDENTIFIER+1;
+		return( 0 );
+	}
+	cf->identifier = cf->ident_buffer + 1;
+	if( cf->LOGNAME ){
+		if( cf->LOGNAME[1] == 0 ){
+			plp_snprintf( cf->error, sizeof( cf->error),
+				"bad LOGNAME line in '%s'", cf->name );
+			return( 1 );
+		}
+		user = cf->LOGNAME+1;
+	}
+	plp_snprintf( cf->ident_buffer, sizeof(cf->ident_buffer),
+		"A%s@", user );
+	len = strlen( cf->ident_buffer );
+	if( cf->FROMHOST ){
+		if( cf->FROMHOST[1] == 0 ){
+			plp_snprintf( cf->error, sizeof( cf->error),
+				"bad FROMHOST line in '%s'", cf->name );
+			return( 1 );
+		}
+		host = cf->FROMHOST+1;
+	}
+	strncat( cf->ident_buffer+len, host, sizeof(cf->ident_buffer)-len );
+	if( (host = strchr( cf->ident_buffer+len, '.' )) ) *host = 0;
+	len = strlen( cf->ident_buffer );
+	plp_snprintf( cf->ident_buffer+len, sizeof(cf->ident_buffer)-len,
+		"+%d", cf->number );
+	return( 0 );
+}
 
 /***************************************************************************
  * Scan_qeueue( char *pathname )
