@@ -10,7 +10,7 @@
  * PURPOSE: perform system dependent initialization
  **************************************************************************/
 
-static char *const _id = "$Id: initialize.c,v 3.0 1996/05/19 04:06:02 papowell Exp $";
+static char *const _id = "$Id: initialize.c,v 3.1 1996/08/25 22:20:05 papowell Exp papowell $";
 
 #include "lp.h"
 #ifdef IS_AUX
@@ -42,5 +42,7 @@ void Initialize()
 #if defined(HAVE_LOCALE_H)
 		setlocale(LC_CTYPE, "");
 #endif
+		/* prepare to catch dead children */
+		Setup_waitpid();
 	}
 }
