@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: link_support.c,v 3.1 1996/07/20 02:42:11 papowell Exp papowell $";
+"$Id: link_support.c,v 3.3 1996/08/25 22:20:05 papowell Exp $";
 
 /***************************************************************************
  * MODULE: Link_support.c
@@ -166,7 +166,7 @@ Remote_port:
 #include "timeout.h"
 #include "setuid.h"
 
-static int reserveport (int low, int high );
+static int reserveport (int low, int high, int type );
 
 /***************************************************************************
  * int Link_open(char *host, int retry, int timeout );
@@ -192,7 +192,7 @@ static int reserveport (int low, int high );
  ***************************************************************************/
 
 /*
- * reserveport(int max_port_no, min_port_no)
+ * reserveport(int max_port_no, min_port_no, type)
  * open a sock and bind to a port, starting at min_port_no, up max_port_no.
  * 
  * NOTE: if min_port_no == maxportno == 0, then we simply open any socket
@@ -232,7 +232,7 @@ int Link_setreuse( int sock )
 	return( option );
 }
 
-static int reserveport(int port_no,  int max_port_no )
+static int reserveport(int port_no,  int max_port_no, int connection_type )
 {
 	struct sockaddr_in sin;
 	int sock, uid, err;
@@ -275,7 +275,7 @@ static int reserveport(int port_no,  int max_port_no )
 		 */
 		if( sock < 0 ){
 			if( UID_root ) (void)To_root();
-			sock = socket(AF_INET, SOCK_STREAM, 0);
+			sock = socket(AF_INET, connection_type, 0);
 			err = errno;
 			if( UID_root ) (void)To_uid( uid );
 			if( sock < 0 ){
@@ -323,7 +323,7 @@ static int reserveport(int port_no,  int max_port_no )
  * 3. makes a connection to the remote host
  */
 
-int getconnection ( char *host, int timeout )
+int getconnection ( char *host, int timeout, int connection_type )
 {
 	struct hostent *hostent;    /* host entry pointer */
 	static int sock;                   /* socket */
@@ -403,7 +403,7 @@ int getconnection ( char *host, int timeout )
 		maxportno = minportno;
 		minportno = i;
 	}
-	sock = reserveport( minportno, maxportno );
+	sock = reserveport( minportno, maxportno, connection_type );
 	if( sock < 0 ){
 		return( sock );
 	}
@@ -515,12 +515,27 @@ int Link_open(char *host, int retry, int timeout )
 	int sock;
 	DBGREM4F(DNW4) ("Link_open: host '%s', retry %d, timeout %d",host,retry,timeout);
 	do {
-		sock = getconnection( host, timeout );
+		sock = getconnection( host, timeout, SOCK_STREAM );
 	} while( sock < 0 && retry-- > 0  );
 	DBGREM4F(DNW4) ("Link_open: socket %d", sock );
 	return( sock );
 }
 
+int Link_open_type(char *host, int retry, int timeout, int port, int connection_type )
+{
+	int sock;
+	int oldport = Destination_port;
+	Destination_port = port;
+	DBGREM4F(DNW4) ("Link_open_type: host '%s', retry %d, timeout %d, port %d, type %d",
+		host,retry,timeout, port, connection_type );
+	do {
+		sock = getconnection( host, timeout, connection_type );
+	} while( sock < 0 && retry-- > 0  );
+	Destination_port = oldport;
+	DBGREM4F(DNW4) ("Link_open_type: socket %d", sock );
+	return( sock );
+}
+
 /***************************************************************************
  * void Link_close( int socket )
  *    closes the link to the remote host
@@ -980,6 +995,7 @@ int Link_read(char *host, int *socket, int timeout,
 	}
 	errno = 0;
 	/* do the read */
+	if( *count < 0 ) *count = 0;
 	len = *count;
 	str = buf;
 	while( len > 0 ){
@@ -998,10 +1014,10 @@ int Link_read(char *host, int *socket, int timeout,
 	}
 	*count = *count - len;
 
-	if( i <= 0 || Alarm_timed_out ){
+	if( (*count == 0) ){
 		if( Alarm_timed_out ){
 			DBGREM4F(DNW4)("Link_read: read %d from '%s' timed out", len, host, i );
-		} else {
+		} else if( i < 0 ) {
 			DBGREM4F(DNW4)("Link_read: read %d from '%s' failed - %s",
 				len, host, i, Errormsg(err) );
 		}
