@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: permissions.c,v 3.1 1996/06/30 17:12:44 papowell Exp $";
+"$Id: permissions.c,v 3.3 1996/08/31 21:11:58 papowell Exp papowell $";
 
 #include "lp.h"
 #include "lp_config.h"
@@ -226,6 +226,7 @@ PAIR(SAMEHOST),
 PAIR(CONTROLLINE),
 PAIR(GROUP),
 PAIR(SERVER),
+PAIR(REMOTEUSER),
 {0}
 };
 
@@ -237,7 +238,7 @@ char *perm_str( int val )
 	if( key->keyword ){
 		return( key->keyword );
 	}
-	plp_snprintf( msg, sizeof(msg), "KEY '%d'", val );
+	plp_snprintf( msg, sizeof(msg), "%d", val );
 	return( msg );
 }
 
@@ -372,12 +373,12 @@ void dump_perm_line( char *title,  struct perm_line *perms )
 	struct perm_val *val;
 	char **s;
 	if( perms ){
-		logDebug( "%sname %d, flag %d", title?title:"",
+		logDebug( "%s name %d, flag %d", title?title:"",
 			perms->name, perms->flag );
 		val = (void *)perms->options.list;
 		for( i = 0; i < perms->options.count; ++i ){
-			logDebug( "  option [%3d] key %d (%s)='%s'",
-				i, val[i].key, perm_str( val[i].key ), val[i].token );
+			logDebug( "  option [%3d] (%s)='%s', values count %d",
+				i, perm_str( val[i].key ), val[i].token, val[i].values.count );
 			s = (void *)val[i].values.list;
 			for( j = 0; j < val[i].values.count; ++j ){
 				logDebug( "    value [%3d] '%s'", j, s[j] );
@@ -411,9 +412,13 @@ void dump_perm_check( char *title,  struct perm_check *check )
 {
 	if( title ) logDebug( "*** perm_check %s ***", title );
 	if( check ){
-		logDebug( "user '%s', host '%s', IP 0x%x",
-			check->user, check->host, check->ip );
-		logDebug( "printer '%s', service '%c'",
+		logDebug(
+			"  user '%s', rmtuser '%s', host '%s', rmthost '%s'",
+			check->user, check->remoteuser, check->host, check->remotehost );
+		logDebug(
+ 			" IP 0x%x,  rmtIP 0x%x, port %d",
+			check->ip, check->remoteip, check->port );
+		logDebug( "  printer '%s', service '%c'",
 			check->printer, check->service );
 	}
 }
@@ -453,9 +458,10 @@ void Free_perms( struct perm_file *perms )
  ***************************************************************************/
 static int match( struct perm_val *val, char *str, int invert );
 static int match_ip( struct perm_val *val, unsigned long ip, int invert );
+static int match_host( struct perm_val *val, char *host, int invert );
 static int match_range( struct perm_val *val, int port, int invert );
 static int match_char( struct perm_val *val, int value, int invert );
-static int groupmatch( struct perm_val *val, char *str, int invert );
+static int match_group( struct perm_val *val, char *str, int invert );
 
 static int Default_perm;
 
@@ -486,15 +492,15 @@ void Init_perms_check()
 		}
 	}
 	Last_default_perm = Default_perm;
-	DEBUG4("Init_perms_check: Default '%s' Last_default_perm %d, Default_perm %d",
-		Default_permission, Last_default_perm, Default_perm );
+	DEBUG4("Init_perms_check: Last_default'%s', Default_perm '%s'",
+		perm_str( Last_default_perm ), perm_str( Default_perm ) );
 }
 
 
 int Perms_check( struct perm_file *perms, struct perm_check *check,
 	struct control_file *cf )
 {
-	int i, j;
+	int i, j, c;
 	int invert = 0;
 	int result, m;
 	struct perm_line *lines, *line;
@@ -511,11 +517,12 @@ int Perms_check( struct perm_file *perms, struct perm_check *check,
 
 			/* skip no entry line */
 			if( line->options.count == 0 ) continue;
+			DEBUG4("Perms_check: current default %s, line %d",
+				perm_str( Last_default_perm ), i );
+			if(Debug>4)dump_perm_line("Perms_check:", line);
 
 			val = (void *)line->options.list;
 			/* clear invert after one test */
-			DEBUG5("Perms_check:line[%d] '%s', current default %s",
-				i, val[0].token, perm_str( Last_default_perm ) );
 			invert = 0;
 			result = 0;
 			m = -1;
@@ -525,25 +532,95 @@ int Perms_check( struct perm_file *perms, struct perm_check *check,
 				} else {
 					invert = 0;
 				}
-				DEBUG6("Perms_check: line[%d] key[%d] current result '%s' checking '%s'",
-					i, j, perm_str( result ), perm_str(val[j].key) );
+				DEBUG5("Perms_check: before key '%s', result = '%s'",
+					 perm_str(val[j].key), perm_str( result ) );
 				switch( val[j].key ){
 				case NOT:
 					invert = 1;
 					continue;
-				case REJECT: result = REJECT; break;
-				case ACCEPT: result = ACCEPT; break;
-				case PORT:
-					m = match_range( &val[j], check->port, invert );
-					break;
-				case USER:
-					m = match( &val[j], check->user, invert );
-					break;
-				case GROUP:
-					m = groupmatch( &val[j], check->user, invert );
-					break;
+				case REJECT: result = REJECT; m = 0; break;
+				case ACCEPT: result = ACCEPT; m = 0; break;
+ 				case USER:
+					m = 1;
+ 					switch (check->service){
+ 					case 'X': break;
+					default:
+ 						m = match( &val[j], check->user, invert );
+ 						break;
+ 					}
+ 					break;
 				case HOST:
-					m = match( &val[j], check->host, invert );
+					m = 1;
+ 					switch (check->service){
+ 					case 'X':
+						m = match_host( &val[j], check->remotehost, invert );
+						break;
+ 					default:
+						m = match_host( &val[j], check->host, invert );
+						break;
+					}
+ 					break;
+				case GROUP:
+					m = 1;
+ 					switch (check->service){
+ 					case 'X': break;
+					default:
+						m = match_group( &val[j], check->user, invert );
+ 						break;
+ 					}
+ 					break;
+				case IP:
+					m = 1;
+ 					switch (check->service){
+					case 'X':
+						m = match_ip( &val[j], check->remoteip, invert );
+						break;
+					default:
+						m = match_ip( &val[j], check->ip, invert );
+						break;
+					}
+					break;
+				case PORT:
+					m = 1;
+ 					switch (check->service){
+ 					case 'X': case 'M': case 'C': case 'S':
+ 						m = match_range( &val[j], check->port, invert );
+ 						break;
+					}
+					break;
+ 				case REMOTEUSER:
+					m = 1;
+ 					switch (check->service){
+ 					case 'X': break;
+ 					case 'M': case 'C': case 'S':
+ 						m = match( &val[j], check->remoteuser, invert );
+						break;
+ 					default:
+ 						m = match( &val[j], check->user, invert );
+ 						break;
+					}
+					break;
+				case REMOTEHOST:
+					m = 1;
+ 					switch (check->service){
+ 					default:
+						m = match_host( &val[j], check->remotehost, invert );
+						break;
+ 					case 'P':
+						m = match_host( &val[j], check->host, invert );
+						break;
+					}
+ 					break;
+				case REMOTEIP:
+					m = 1;
+					switch( check->service ){
+					default:
+						m = match_ip( &val[j], check->remoteip, invert );
+						break;
+					case 'P':
+						m = match_ip( &val[j], check->ip, invert );
+						break;
+					}
 					break;
 				case CONTROLLINE:
 					/* check to see if we have control line */
@@ -551,50 +628,85 @@ int Perms_check( struct perm_file *perms, struct perm_check *check,
 					if( cf == 0 ){
 						m = 1;
 					} else {
-						s = cf->capoptions[val[j].token[0] - 'A'];
-						if( s ) ++s;
-						m = match( &val[j], s, invert );
+						c = val[j].token[0];
+						if( !isupper(c) ){
+							fatal( LOG_ERR, "Perms_check: bad CONTROLLINE '%s'",
+								val[j].token );
+						}
+						/* match the control file line */
+						if( (s = cf->capoptions[c - 'A']) ){
+							/* s points to start of line */
+							m = match( &val[j], s, invert );
+						}
 					}
 					break;
-				case REMOTEHOST:
-					m = match( &val[j], check->remotehost, invert );
-					break;
-				case IP:
-					m = match_ip( &val[j], check->ip, invert );
-					break;
-				case REMOTEIP:
-					m = match_ip( &val[j], check->remoteip, invert );
-					break;
 				case PRINTER:
-					m = match( &val[j], check->printer, invert );
+					m = 1;
+ 					switch (check->service){
+ 					case 'X': break;
+					default:
+						m = match( &val[j], check->printer, invert );
+						break;
+					}
 					break;
 				case SERVICE:
 					m = match_char( &val[j], check->service, invert );
 					break;
 				case FORWARD:
-					/* check succeeds if REMOTEIP != IP */
-					m = !(check->ip != check->remoteip);
+					m = 1;
+ 					switch (check->service){
+					default: break;
+					case 'R': case 'Q': case 'M': case 'C': case 'S':
+						/* FORWARD check succeeds if REMOTEIP != IP */
+						m = !(check->ip != check->remoteip);
+						if( invert ) m = !m;
+						DEBUG8(
+						"Perms_check: FORWARD IP '0x%x' != rmtIP '0%x', rslt %d",
+						check->ip, check->remoteip, m );
+						break;
+					}
 					break;
 				case SAMEHOST:
-					/* check succeeds if REMOTEIP == IP */
-					m = !(check->ip == check->remoteip);
-					DEBUG8(
-						"Perms_check: SAMEHOST ip 0x%x == remote 0x%x, rslt %d",
+					m = 1;
+ 					switch (check->service){
+					default: break;
+					case 'R': case 'Q': case 'M': case 'C': case 'S':
+						/* SAMEHOST check succeeds if REMOTEIP == IP */
+						m = !(check->ip == check->remoteip);
+						if( invert ) m = !m;
+						DEBUG8(
+						"Perms_check: SAMEHOST IP '0x%x' == rmtIP '0%x', rslt %d",
 						check->ip, check->remoteip, m );
-					break;
-				case SERVER:
-					/* check succeeds if server and server IP == IP */
-					m = !(check->ip == ntohl(HostIP) );
-					DEBUG8(
-						"Perms_check: SERVER ip 0x%x == HostIP 0x%x, rslt %d",
-						check->ip, ntohl(HostIP), m );
+						break;
+					}
 					break;
 				case SAMEUSER:
-					/* check succeeds if remoteuser == user */
-					m = safestrcmp( check->user, check->remoteuser );
-					DEBUG8(
+					m = 1;
+ 					switch (check->service){
+					default: break;
+					case 'Q': case 'M': case 'C': case 'S':
+						/* check succeeds if remoteuser == user */
+						m = safestrcmp( check->user, check->remoteuser );
+						if( invert ) m = !m;
+						DEBUG8(
 						"Perms_check: SAMEUSER '%s' == remote '%s', rslt %d",
 						check->user, check->remoteuser, m );
+						break;
+					}
+					break;
+				case SERVER:
+					m = 1;
+ 					switch (check->service){
+					default: break;
+					case 'R': case 'Q': case 'M': case 'C': case 'S':
+						/* check succeeds if server and server IP == IP */
+						m = !(check->ip == ntohl(HostIP) );
+						if( invert ) m = !m;
+						DEBUG8(
+						"Perms_check: SERVER ip 0x%x == HostIP 0x%x, rslt %d",
+							check->ip, ntohl(HostIP), m );
+						break;
+					}
 					break;
 				case DEFAULT:
 					++j;
@@ -623,9 +735,10 @@ int Perms_check( struct perm_file *perms, struct perm_check *check,
 					}
 					goto next_line;
 				}
-				DEBUG6("Perms_check: last check status %d", m );
 				/* if status is non-zero, test failed */
-				if( m > 0 ){
+				DEBUG6("Perms_check: key '%s', result after '%s', match %d",
+					 perm_str(val[j].key), perm_str( result ), m );
+				if( m != 0 ){
 					goto next_line;
 				}
 			}
@@ -641,7 +754,7 @@ int Perms_check( struct perm_file *perms, struct perm_check *check,
 		next_line: ;
 		}
 	}
-	DEBUG4("Perms_check: no match" );
+	DEBUG4("Perms_check: result 0 (no match)" );
 	return( 0 );
 }
 
@@ -663,7 +776,7 @@ static int match( struct perm_val *val, char *str, int invert )
  	int i;
  	DEBUG8("match: str '%s', %d entries to check", str, val->values.count );
  	list = (void *)val->values.list;
- 	for( i = 0; result && i < val->values.count; ++i ){
+ 	for( i = 0; str && result && i < val->values.count; ++i ){
  		/* now do the match */
  		result = Globmatch( list[i], str );
 		DEBUG8("match: list[%d]='%s', result %d", i, list[i],  result );
@@ -674,7 +787,42 @@ static int match( struct perm_val *val, char *str, int invert )
 }
 
 /***************************************************************************
- * static int match_ip( struct perm_val *val, char *str );
+ * static int match_host( struct perm_val *val, char *host );
+ *  returns 1 on failure, 0 on success
+ *  - match the string against the list of options
+ *    options are glob type regular expressions;  we implement this
+ *    currently using the most crude of pattern matching
+ *  - if string is null or pattern list is null, then match fails
+ *    if both are null, then match succeeds
+ ***************************************************************************/
+
+static int match_host( struct perm_val *val, char *host, int invert )
+{
+ 	char **list;
+ 	int result = 1;
+ 	int i;
+ 	DEBUG8("match: host '%s', %d entries to check", host, val->values.count );
+ 	list = (void *)val->values.list;
+ 	for( i = 0; host && result && i < val->values.count; ++i ){
+ 		/* now do the match */
+		if( list[i][0] == '@' ) {	/* look up host in netgroup */
+#ifdef HAVE_INNETGR
+			result = !innetgr( list[i]+1, host, NULL, NULL );
+#else /* HAVE_INNETGR */
+			DEBUG8("match: no innetgr() call, netgroups not permitted");
+#endif /* HAVE_INNETGR */
+		} else {
+	 		result = Globmatch( list[i], host );
+		}
+		DEBUG8("match: list[%d]='%s', result %d", i, list[i],  result );
+	}
+	if( invert ) result = !result;
+ 	DEBUG8("match: host '%s' final result %d", host, result );
+	return( result );
+}
+
+/***************************************************************************
+ * static int match_ip( struct perm_val *val )
  * check the IP address and mask
  * entry has the format:  IPADDR/MASK, mask is x.x.x.x or n (length)
  ***************************************************************************/
@@ -819,7 +967,7 @@ static int match_char( struct perm_val *val, int value, int invert )
 
 
 /***************************************************************************
- * static int groupmatch( struct perm_val *val, char *str );
+ * static int match_group( struct perm_val *val, char *str );
  *  returns 1 on failure, 0 on success
  *  - get the UID for the named user
  *  - scan the listed groups to see if there is a group
@@ -827,14 +975,14 @@ static int match_char( struct perm_val *val, int value, int invert )
  ***************************************************************************/
 static int ingroup( char *group, char *user );
 
-static int groupmatch( struct perm_val *val, char *str, int invert )
+static int match_group( struct perm_val *val, char *str, int invert )
 {
  	char **list;
  	int result = 1;
  	int i;
- 	DEBUG8("groupmatch: str '%s'", str );
+ 	DEBUG8("match_group: str '%s'", str );
  	list = (void *)val->values.list;
- 	for( i = 0; result && i < val->values.count; ++i ){
+ 	for( i = 0; str && result && i < val->values.count; ++i ){
  		/* now do the match */
  		result = ingroup( list[i], str );
 	}
@@ -857,9 +1005,11 @@ static int ingroup( char *group, char *user )
 	struct passwd *pwent;
 	char **members;
 	int result = 1;
-	int ingroup;
 
-	DEBUG8("ingroup: checking for membership in group '%s'", group);
+	DEBUG8("ingroup: checking '%s' for membership in group '%s'", user, group);
+	if( group == 0 || user == 0 ){
+		return( result );
+	}
 	/* first try getgrnam, see if it is a group */
 	if( (grent = getgrnam( group )) ){
 		DEBUG8("ingroup: group id: %d\n", grent->gr_gid);
@@ -876,8 +1026,7 @@ static int ingroup( char *group, char *user )
 		while( result && (grent = getgrent()) ){
 			DEBUG8("ingroup: group name '%s'", grent->gr_name);
 			/* now do match against group */
-			ingroup = Globmatch( group, grent->gr_name );
-			if( ingroup == 0 ){
+			if( Globmatch( group, grent->gr_name ) == 0 ){
 				DEBUG8("ingroup: found '%s'", grent->gr_name);
 				for( members = grent->gr_mem; result && *members; ++members ){
 					DEBUG8("ingroup: member '%s'", *members);
@@ -886,6 +1035,17 @@ static int ingroup( char *group, char *user )
 			}
 		}
 		endgrent();
+	} else if( group[0] == '@' ) {	/* look up user in netgroup */
+#ifdef HAVE_INNETGR
+		if( !innetgr( group+1, NULL, user, NULL ) ) {
+			DEBUG8( "ingroup: user %s NOT in netgroup %s", user, group+1 );
+		} else {
+			DEBUG8( "ingroup: user %s in netgroup %s", user, group+1 );
+			result = 0;
+		}
+#else /* HAVE_INNETGR */
+		DEBUG8( "ingroup: no innetgr() call, netgroups not permitted" );
+#endif /* HAVE_INNETGR */
 	}
 	DEBUG8("ingroup: result: %d\n", result );
 	return( result );
