@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: printcap.c,v 3.3 1996/07/20 02:42:11 papowell Exp papowell $";
+"$Id: printcap.c,v 3.7 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lp.h"
 #include "printcap.h"
@@ -405,7 +405,7 @@ char *Filter_read( char *name, struct malloc_list *list, char *filter )
 	DEBUG8("Filter_read: pipe [%d,%d]", pipes[0],pipes[1] );
 	/* create the printcap info process */
 	Make_filter( 'f',(void *)0,&Pr_fd_info,filter,1, 0, pipes[1],
-		(void *)0, (void *)0, 0 );
+		(void *)0, (void *)0, 0, 0 );
 	DEBUG8("Filter_read: filter pid %d, input fd %d, sending '%s'",
 		Pr_fd_info.pid, Pr_fd_info.input, name?name:"<NULL>" );
 	close( pipes[1] );
@@ -1873,6 +1873,7 @@ static struct keywords long_pc_vars[]
 		 /*  error log file (servers, filters and prefilters) */
 	{ "lk", FLAG_K,  &Lock_it },
 		 /* lock the IO device */
+	{ "logger_destination",  STRING_K,  &Logger_destination, 0, 1 },
 	{ "longnumber",  FLAG_K,  &Long_number, 0, 1 },
 		 /*  use long job number when a job is submitted */
 	{ "lp",  STRING_K,  &Lp_device },
@@ -1907,6 +1908,8 @@ static struct keywords long_pc_vars[]
 		 /*  put queue name in control file */
 	{ "rm",  STRING_K,  &RemoteHost },
 		 /*  remote-queue machine (hostname) (with rm) */
+	{ "router",  STRING_K,  &Routing_filter },
+		 /*  routing filter, returns destinations */
 	{ "rp",  STRING_K,  &RemotePrinter },
 		 /*  remote-queue printer name (with rp) */
 	{ "rs",  INTEGER_K,  &Rescan_time },
@@ -1915,6 +1918,8 @@ static struct keywords long_pc_vars[]
 		 /*  number of times to try printing or transfer (0=infinite) */
 	{ "rw",  FLAG_K,  &Read_write },
 		 /*  open the printer for reading and writing */
+	{ "save_when_done",  FLAG_K,  &Save_when_done, 0, 1 },
+		 /*  save job when done */
 	{ "sb",  FLAG_K,  &Short_banner },
 		 /*  short banner (one line only) */
 	{ "sc",  FLAG_K,  &Suppress_copies },
@@ -1941,15 +1946,19 @@ static struct keywords long_pc_vars[]
 		 /*  stty commands to set output line characteristics */
 	{ "tr",  STRING_K,  &Trailer_on_close },
 		 /*  trailer string to print when queue empties */
+	{ "translate_format",  STRING_K,  &Xlate_format },
+		 /*  translate format from one to another - similar to tr(1) utility */
 	{ "ty",  STRING_K,  &Stty_command },
 		 /*  stty commands to set output line characteristics */
+	{ "use_identifier",  FLAG_K,  &Use_identifier, 0, 1 },
+		 /*  put identifier in control file */
 	{ "use_shorthost",  INTEGER_K,  &Use_shorthost, 0, 1 },
 		 /*  Use short hostname for lpr control and data file names */
 	{ "xc",  INTEGER_K,  &Clear_local_bits },
 		 /*  if lp is a tty, clear local mode bits (see ty) */
 	{ "xs",  INTEGER_K,  &Set_local_bits },
 		 /*  like `xc' but set bits (see ty) */
-	{ "xt",  FLAG_K,  &Check_for_nonprintable },
+	{ "xt",  FLAG_K,  &Check_for_nonprintable, 0, 1 },
 		/*  formats to check for printable files */
 	{ "xu",  STRING_K,  &Local_permission_file },
 		 /*  additional permissions file for this queue */
