@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: printjob.c,v 3.4 1996/07/27 18:15:32 papowell Exp $";
+"$Id: printjob.c,v 3.8 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lp.h"
 #include "lp_config.h"
@@ -106,6 +106,16 @@ static int setup_exit;
 static int Print_banner( char *cmd, struct control_file *cfp,
 	int timeout, int ff, char *str, struct pc_used *pc_used );
 
+static char *Pr_temp;
+
+static void Remove_pr_temp()
+{
+	if( Pr_temp && *Pr_temp ){
+		unlink(Pr_temp);
+		Pr_temp = 0;
+	}
+}
+
 int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 {
 	int FF_len;				/* FF information length */
@@ -124,18 +134,23 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	char *pname;			/* pname */
 	char *s;				/* Sigh... */
 	int do_banner;			/* do a banner */
+	int pfd;				/* pr program output */
 
 	DEBUG2("Print_job: '%s', Use_queuename %d", cfp->name, Use_queuename );
 
 	if( Use_queuename && cfp->QUEUENAME == 0 && Printer ){
-		static char buffer[M_QUEUENAME];
+		char buffer[M_QUEUENAME];
 		plp_snprintf( buffer, sizeof(buffer), "Q%s", Printer );
-		cfp->QUEUENAME = buffer;
+		cfp->QUEUENAME = Prefix_job_line( cfp, buffer );
+	}
+	if( Use_identifier && cfp->IDENTIFIER == 0 ){
+		cfp->IDENTIFIER = Prefix_job_line( cfp, cfp->identifier-1 );
 	}
 
 	/* we will not try to do anything fancy on exit except kill filters */
 	if( setup_exit == 0 ){
 		register_exit( (exit_ret)Print_close, (void *)1 );
+		register_exit( (exit_ret)Remove_pr_temp, (void *)1 );
 		setup_exit = 1;
 	}
 
@@ -167,7 +182,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	 * }
 	 */
 
-	setstatus(NORMAL,"printing '%s'",cfp->name);
+	setstatus(cfp,"printing '%s'",cfp->name);
 	if( Print_open( &Device_fd_info, cfp, Connect_timeout, Connect_interval,
 		Connect_grace, Connect_try, pc_used, Accounting_port ) < 0 ){
 		/* we failed to open device */
@@ -180,7 +195,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	if( OF_Filter ){
 		/* we need to create an OF filter */
 		Make_filter( 'o', cfp, &OF_fd_info, OF_Filter, 0, 0,
-			Device_fd_info.input, pc_used, (void *)0, Accounting_port );
+			Device_fd_info.input, pc_used, (void *)0, Accounting_port, Logger_destination != 0 );
 	} else {
 		OF_fd_info.input = Device_fd_info.input;
 		OF_fd_info.pid = 0;
@@ -198,10 +213,10 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	 * do accounting at start
 	 */
 	if( Accounting_start && Local_accounting ){
-		setstatus(NORMAL,"accounting at start '%s'", cfp->name );
+		setstatus(cfp,"accounting at start '%s'", cfp->name );
 		i = Do_accounting( 0, Accounting_start, cfp, Send_timeout, pc_used, OF_fd_info.input );
 		if( i ){
-			setstatus(NORMAL,"accounting failed at start '%s' - %s", cfp->name, Server_status(i) );
+			setstatus(cfp,"accounting failed at start '%s' - %s", cfp->name, Server_status(i) );
 			if( i != JFAIL && i != JABORT && i != JREMOVE ){
 				i = JFAIL;
 			}
@@ -212,11 +227,11 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 
 	/* Leader_on_open -> of_fd; */
 	if( leader_len ){
-		setstatus(NORMAL,"printing '%s', sending leader",cfp->name);
+		setstatus(cfp,"printing '%s', sending leader",cfp->name);
 		i = Print_string( &OF_fd_info, leader_str, leader_len,
 			Send_timeout );
 		if( i ){
-			setstatus(NORMAL,"printing '%s', error sending leader",cfp->name);
+			setstatus(cfp,"printing '%s', error sending leader",cfp->name);
 			n = Close_filter( &OF_fd_info, Send_timeout );
 			if( n ) i = n;
 			Errorcode = i;
@@ -226,11 +241,11 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 
 	/* FF_on_open     -> of_fd; */
 	if( FF_on_open && FF_len ){
-		setstatus(NORMAL,"printing '%s', sending FF on open",cfp->name);
+		setstatus(cfp,"printing '%s', sending FF on open",cfp->name);
 		i = Print_string( &OF_fd_info, FF_str, FF_len,
 			Send_timeout );
 		if( i ){
-			setstatus(NORMAL,"printing '%s', error sending FF on open",cfp->name);
+			setstatus(cfp,"printing '%s', error sending FF on open",cfp->name);
 			n = Close_filter( &OF_fd_info, Send_timeout );
 			if( n ) i = n;
 			Errorcode = i;
@@ -260,7 +275,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 
 	/* now we have a banner, is it at start or end? */
 	if( do_banner && ( Banner_start || !Banner_last ) ){
-		setstatus(NORMAL,"printing '%s', printing banner on open",cfp->name);
+		setstatus(cfp,"printing '%s', printing banner on open",cfp->name);
 		s = 0;
 		DEBUG6( "Print_job: Banner_start '%s', Banner_printer '%s', Default '%s'",
 			Banner_start, Banner_printer, Default_banner_printer );
@@ -271,7 +286,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 		if( Banner_start && *Banner_start ) s = Banner_start;
 		i = Print_banner( s, cfp, Send_timeout, FF_len, FF_str, pc_used );
 		if( i ){
-			setstatus(NORMAL,"printing '%s', error printing banner on open",cfp->name);
+			setstatus(cfp,"printing '%s', error printing banner on open",cfp->name);
 			Errorcode = i;
 			cleanup( 0 );
 		}
@@ -287,7 +302,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 
 	if( OF_fd_info.input != Device_fd_info.input
 		&& (i = of_stop( &OF_fd_info, Send_timeout ) ) ){
-		setstatus(NORMAL,"printing '%s', error suspending OF filter",cfp->name);
+		setstatus(cfp,"printing '%s', error suspending OF filter",cfp->name);
 		n = Close_filter( &OF_fd_info, Send_timeout );
 		if( n ) i = n;
 		Errorcode = i;
@@ -317,7 +332,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 
 		if( i && !No_FF_separator && FF_len ){
 			/* FF separator -> of_fd; */
-			setstatus(NORMAL,"printing '%s', FF separator ",cfp->name);
+			setstatus(cfp,"printing '%s', FF separator ",cfp->name);
 			n = 0;
 			if( OF_fd_info.input != Device_fd_info.input ){
 				n = of_start( &OF_fd_info );
@@ -337,18 +352,18 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 			}
 		}
 
-		setstatus(NORMAL,"printing job '%s data file '%s', format '%c'",
+		setstatus(cfp,"printing job '%s data file '%s', format '%c'",
 			cfp->name, data_file->openname, data_file->format );
 		pname = Add_path( SDpathname, data_file->openname );
 		fd = Checkread( pname, &statb );
 		if( fd < 0 ){
-			setstatus(NORMAL,"cannot open data file '%s'",data_file->openname);
+			setstatus(cfp,"cannot open data file '%s'",data_file->openname);
 			fatal( LOG_ERR, "Print_job: job '%s', cannot open data file '%s'",
 				cfp->name, data_file->openname );
 		}
 		if( (statb.st_size != data_file->statb.st_size)
 			|| (statb.st_ctime != data_file->statb.st_ctime) ){
-			setstatus(NORMAL,"data file '%s' changed during printing",
+			setstatus(cfp,"data file '%s' changed during printing",
 				data_file->openname);
 			Errorcode = JABORT;
 			fatal( LOG_CRIT,
@@ -356,33 +371,77 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 				data_file->openname );
 		}
 		/*
-		 * now we check to see if there is an input filter
+		 * check for PR to be used
 		 */
-		filter = 0;
-		switch( c = data_file->format ){
-		case 'p':
-			if( Pr_program ){
-				static char pr_filter[SMALLBUFFER];
-				pr_filter[0] = 0;
-				safestrncat( pr_filter, Pr_program );
-				safestrncat( pr_filter, " -w%w -l%l" );
-				if( cfp->PRTITLE ) safestrncat( pr_filter, " -h %T" );
-				filter = pr_filter;
-			} else {
-				setstatus(NORMAL,"no 'p' format filter available" );
+		c = data_file->format;
+		if( 'p' == c ){
+			char pr_filter[SMALLBUFFER];
+			if( Pr_program == 0 || *Pr_program  == 0 ){
+				setstatus(cfp,"no 'p' format filter available" );
 				Errorcode = JREMOVE;
 				fatal( LOG_ERR, "Print_job: no '-p' formatter for '%s'",
 					c, data_file->openname );
 			}
-			break;
-		case 'f': case 'l':
+			pr_filter[0] = 0;
+			safestrncat( pr_filter, Pr_program );
+			safestrncat( pr_filter, " $w $l" );
+			if( cfp->PRTITLE ) safestrncat( pr_filter, " -h $-T" );
+			/* we need to open a file and process the job */
+			s = strrchr( pname, '/' );
+			if( s == 0 ){
+				Errorcode = JABORT;
+				fatal( LOG_ERR, "Print_job: bad pathname to job file",
+					pname );
+			}
+			s[1] = 'B';
+			Pr_temp = pname;
+			/* make a temp file */
+			pfd = Checkwrite( pname, &statb, O_RDWR, 1, 0 );
+			DEBUG8( "Print_job: pr temp file '%s', fd %d", pname, pfd );
+			if( ftruncate( pfd, 0 )  < 0 ){
+				Errorcode = JABORT;
+				logerr_die( LOG_ERR, "Print_job: ftruncate of %s failed",
+					pname );
+			}
+			/* make the pr program */
+			Make_filter( 'f', cfp, &Pr_fd_info, pr_filter, 1, 0,
+				pfd, pc_used, data_file, Accounting_port,
+				Logger_destination != 0 );
+			/* filter the data file */
+			err = Print_copy(cfp, fd, &data_file->statb, &Pr_fd_info,
+				Send_timeout, 1);
+			n = Close_filter( &Pr_fd_info, Send_timeout );
+			DEBUG8( "Print_job: pr filter status  %d", n );
+			if( n ){
+				Errorcode = n;
+				cleanup(0);
+			}
+			/* unlink the temporary file */
+			Remove_pr_temp();
+			/* save the file descriptor */
+			if( dup2( pfd, fd ) < 0 ){
+				Errorcode = JABORT;
+				logerr_die( LOG_ERR, "Print_job: dup2 failed" );
+			}
+			if( lseek( fd, 0, SEEK_SET ) == (off_t)(-1) ){
+				Errorcode = JABORT;
+				logerr_die( LOG_ERR, "Print_job: lseek failed" );
+			}
+		}
+		/*
+		 * now we check to see if there is an input filter
+		 */
+		filter = 0;
+		switch( c ){
+		case 'p': case 'f': case 'l':
 			filter = IF_Filter;
+			c = 'f';
 			break;
 		default:
 			filter = Find_filter( c, pc_used );
 			if( filter == 0 ){
 				Errorcode = JREMOVE;
-				setstatus(NORMAL,"no '%c' format filter available", c );
+				setstatus(cfp,"no '%c' format filter available", c );
 				fatal( LOG_ERR, "Print_job: cannot find filter '%c' for '%s'",
 					c, data_file->openname );
 			}
@@ -390,14 +449,14 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 		/* hook filter up to the device file descriptor */
 		if( filter ){
 			Make_filter( c, cfp, &XF_fd_info, filter, 0, 0,
-				Device_fd_info.input, pc_used, data_file, Accounting_port );
+				Device_fd_info.input, pc_used, data_file, Accounting_port, Logger_destination != 0 );
 		} else {
 			XF_fd_info.input = Device_fd_info.input;
 			XF_fd_info.pid = 0;
 		}
 		/* send job to printer - we get the error status of filter if any */
 		n = 0;
-		err = Print_copy(fd, &data_file->statb, &XF_fd_info, Send_timeout, 1);
+		err = Print_copy(cfp, fd, &data_file->statb, &XF_fd_info, Send_timeout, 1);
 		(void)close(fd);
 		/* close Xf */
 		n = Close_filter( &XF_fd_info, Send_timeout );
@@ -441,7 +500,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	if( do_banner && ( Banner_end || Banner_last ) ){
 		if( !No_FF_separator && FF_len ){
 			/* FF befor banner     -> of_fd; */
-			setstatus(NORMAL,"printing '%s', FF separator ",cfp->name);
+			setstatus(cfp,"printing '%s', FF separator ",cfp->name);
 			DEBUG8( "Print_job: printing FF separator" );
 			i = Print_string( &OF_fd_info, FF_str, FF_len,
 				Send_timeout );
@@ -451,7 +510,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 				cleanup( 0 );
 			}
 		}
-		setstatus(NORMAL,"printing '%s', printing banner on close",cfp->name);
+		setstatus(cfp,"printing '%s', printing banner on close",cfp->name);
 		s = 0;
 		DEBUG6( "Print_job: Banner_end '%s', Banner_printer '%s', Default '%s'",
 			Banner_end, Banner_printer, Default_banner_printer );
@@ -473,7 +532,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	 * FF_on_close     -> of_fd;
 	 */ 
 	if( FF_on_close && FF_len ){
-		setstatus(NORMAL,"printing '%s', sending FF on close",cfp->name);
+		setstatus(cfp,"printing '%s', sending FF on close",cfp->name);
 		DEBUG8( "Print_job: printing FF on close" );
 		i = Print_string( &OF_fd_info, FF_str, FF_len, Send_timeout );
 		if( i ){
@@ -487,7 +546,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	 * Trailer_on_close -> of_fd;
 	 */ 
 	if( trailer_len ){
-		setstatus(NORMAL,"printing '%s', sending trailer",cfp->name);
+		setstatus(cfp,"printing '%s', sending trailer",cfp->name);
 		DEBUG8( "Print_job: printing trailer" );
 		i = Print_string( &OF_fd_info, trailer_str,trailer_len,
 			Send_timeout);
@@ -503,12 +562,12 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	 * do accounting at end
 	 */
 	if( Accounting_end && Local_accounting ){
-		setstatus(NORMAL,"accounting at end '%s'", cfp->name );
+		setstatus(cfp,"accounting at end '%s'", cfp->name );
 		DEBUG8( "Print_job: accounting at end" );
 		i = Do_accounting( 1, Accounting_end, cfp, Send_timeout, pc_used,
 			OF_fd_info.input );
 		if( i ){
-			setstatus(NORMAL,"accounting failed at end '%s'", cfp->name );
+			setstatus(cfp,"accounting failed at end '%s'", cfp->name );
 			DEBUG8( "Print_job: accounting at end FAILED, status 0x%x", i );
 			if( i != JFAIL && i != JABORT && i != JREMOVE ){
 				i = JFAIL;
@@ -521,7 +580,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 	/*
 	 * close( of_fd );
 	 */
-	setstatus(NORMAL,"job '%s', closing device",cfp->name);
+	setstatus(cfp,"job '%s', closing device",cfp->name);
 	n = Close_filter( &OF_fd_info, Send_timeout );
 	if( n ){
 		DEBUG8( "Print_job: close OF filter FAILED, status 0x%x", n );
@@ -549,7 +608,7 @@ int Print_job( struct control_file *cfp, struct pc_used *pc_used )
 		Errorcode = JSUCC;
 	}
 
-	setstatus(NORMAL,"job '%s' finished ",cfp->name);
+	setstatus(cfp,"job '%s' finished ",cfp->name);
 	return( Errorcode );
 }
 
@@ -573,7 +632,7 @@ static int Fix_str( char **copy, char *str )
 	}
 	DEBUG9("Fix_str: '%s'", str );
 	for( len = 0, s = *copy, t = str; t && (c = (s[len] = *t++)); ++len ){
-		DEBUG9("Fix_str: char '%c', len %d", c, len );
+		/* DEBUG9("Fix_str: char '%c', len %d", c, len ); */
 		if( c == '\\' && (c = *t) != '\\' ){
 			/* check for \nnn */
 			if( isdigit( c ) ){
@@ -595,7 +654,7 @@ static int Fix_str( char **copy, char *str )
 				++t;
 			}
 		}
-		DEBUG9("Fix_str: result '0x%x', len %d", s[len], len );
+		/* DEBUG9("Fix_str: result '0x%x', len %d", s[len], len ); */
 	}
 	if( Debug > 8 ){
 		logDebug( "Fix_str: str '%s' -> ", str );
@@ -646,7 +705,7 @@ static int Print_banner(
 
  	if( !Short_banner && banner_printer && *banner_printer ){
 		Make_filter( 'f', cfp, &Pr_fd_info, banner_printer, 0, 0,
-			OF_fd_info.input, pc_used, (void *)0, Accounting_port );
+			OF_fd_info.input, pc_used, (void *)0, Accounting_port, Logger_destination != 0 );
 	} else {
 		Pr_fd_info.input = OF_fd_info.input;
 		Pr_fd_info.pid = 0;
@@ -667,7 +726,7 @@ static int Print_banner(
 	}
 	if( !No_FF_separator && FF_len ){
 		/* FF after banner     -> of_fd; */
-		setstatus(NORMAL,"printing '%s', sending FF after banner",cfp->name);
+		setstatus(cfp,"printing '%s', sending FF after banner",cfp->name);
 		i = Print_string( &OF_fd_info, FF_str, FF_len,
 			Send_timeout );
 		if( i ){
@@ -691,7 +750,7 @@ void Setup_accounting( struct control_file *cfp, struct pc_used *pc_used )
 		/* first try to make the filter */
 		Accounting_file = 0;
 		Make_filter( 'f', cfp, &Af_fd_info, s, 0, 0,
-			0, pc_used, (void *)0, 0 );
+			0, pc_used, (void *)0, 0, Logger_destination != 0 );
 		Accounting_port = Af_fd_info.input;
 	} else if( (s = strchr( s, '%' )) ){
 		/* now try to open a connection to a server */
@@ -758,14 +817,14 @@ int Do_accounting( int end, char *command, struct control_file *cfp,
 		;
 	} else if( *command == '|' ){
 		Make_filter( 'f', cfp, &As_fd_info, command, 0, 0,
-			filter_out, pc_used, (void *)0, Accounting_port );
+			filter_out, pc_used, (void *)0, Accounting_port, Logger_destination != 0 );
 		err = Close_filter( &As_fd_info, timeout );
 		DEBUG6("Do_accounting: filter exit status %s", Server_status(err) );
 		if( err ){
 			plp_snprintf( cfp->error, sizeof(cfp->error),
 				"accounting check at %s failed - status %s", end?"end":"start",
 				Server_status( err ) );
-			setstatus( NORMAL,  cfp->error );
+			setstatus( cfp,  cfp->error );
 			Set_job_control( cfp );
 		}
 	} else if( Accounting_port > 0 ){
