@@ -12,7 +12,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: sendjob.c,v 3.3 1996/07/20 15:10:53 papowell Exp $";
+"$Id: sendjob.c,v 3.7 1996/09/09 14:24:41 papowell Exp papowell $";
 
 #include "lp.h"
 #include "lp_config.h"
@@ -66,7 +66,7 @@ int Send_job(
 	char *printer,	- name of printer
 	char *host,		- remote host name
 	struct dpathname *p - pathname to spool directory
-	struct control_file *cf,	- control file
+	struct control_file *cfp,	- control file
 	int connect_try,	- number of times to retry sending/opening connection
 						- negative is infinite
 	int connect_timeout,	- timeout on making connection
@@ -82,10 +82,10 @@ int Send_job(
  **************************************************************************/
 int Send_files( char *host, char *printer,
 	struct dpathname *dpath,
-	int *sock, struct control_file *cf, int transfer_timeout,
+	int *sock, struct control_file *cfp, int transfer_timeout,
 	struct pc_used *pc_used );
 
-int Send_job( char *printer, char *host, struct control_file *cf,
+int Send_job( char *printer, char *host, struct control_file *cfp,
 	struct dpathname *dpath,
 	int max_try, int connect_timeout, int connect_interval,
 	int transfer_timeout, struct pc_used *pc_used )
@@ -99,24 +99,21 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 	char *line;
 	int err;
  
-	if( Verbose || Debug > 4 ){
-		logDebug( "connect retry %d, timeout %d, interval %d, transfer timeout %d",
-			max_try, connect_timeout, connect_interval,
-			transfer_timeout );
-	}
-	if( Fix_control( cf, &line ) ){
-		log( LOG_ERR, "Send_job: control file '%s' bad line '%s'", cf->name, line );
+
+	/* fix up the control file */
+
+	if( Fix_control( cfp, &line ) ){
+		log( LOG_ERR, "Send_job: control file '%s' bad line '%s'", cfp->name, line );
 		return( JABORT );
 	}
 
-
 	/*
 	 * do accounting at start
 	 */
 	if( pc_used && Accounting_remote ){
-		Setup_accounting( cf, pc_used );
+		Setup_accounting( cfp, pc_used );
 		if( Accounting_start ){
-			i = Do_accounting( 0, Accounting_start, cf, Send_timeout, pc_used, 0 );
+			i = Do_accounting( 0, Accounting_start, cfp, Send_timeout, pc_used, 0 );
 			if( i ){
 				Errorcode = i;
 				cleanup( 0 );
@@ -125,10 +122,10 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 	}
 
 	DEBUG4("Send_job: '%s'->'%s@%s',connect(retry %d,timeout %d,interval %d)",
-		cf->name, printer, host, max_try, connect_timeout,
+		cfp->name, printer, host, max_try, connect_timeout,
 		connect_interval );
 
-	setstatus( NORMAL, "start job '%s' transfer to %s@%s", cf->name,
+	setstatus( cfp, "start job '%s' transfer to %s@%s", cfp->name,
 		 printer, host );
 
 	if( connect_interval > 0 ){
@@ -141,16 +138,16 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 	do {
 		/* we pause on later tries */
 		if( attempt > 0 ){
-			setstatus( NORMAL,
+			setstatus( cfp,
 			"sending job '%s' to '%s@%s', connect attempt %d failed, sleeping %d",
-				cf->name, printer, host, attempt, n );
+				cfp->name, printer, host, attempt, n );
 			sleep( n );
 			n += connect_interval;
 			if( n > 60 ) n = 60;
 		}
 		++attempt;
 
-		setstatus( NORMAL,
+		setstatus( cfp,
 			"connection to %s@%s attempt %d", printer, host, attempt );
 
 		errno = 0;
@@ -162,7 +159,7 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 			plp_snprintf(msg,sizeof(msg)-2,
 				"connection to %s@%s failed - %s", printer, host,
 					Errormsg(err) );
-			setstatus( NORMAL, msg );
+			setstatus( cfp, msg );
 			if( Interactive ){
 				/* send it directly to the error output */
 				strcat(msg,"\n");
@@ -176,7 +173,7 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 		} else {
 			/* send job to the LPD server for the printer */
 			status = Send_files( host, printer,
-				dpath, &sock, cf, transfer_timeout, pc_used );
+				dpath, &sock, cfp, transfer_timeout, pc_used );
 			/* we read any other information that is sent */
 			if( status != 0 ){
 				do{
@@ -205,7 +202,7 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 	if( status ){
 		plp_snprintf(msg, sizeof(msg)-2,
 			"job '%s' transfer to %s@%s failed after %d attempts",
-			cf->name, printer, host, attempt );
+			cfp->name, printer, host, attempt );
 		if( Interactive ){
 			strcat(msg,"\n");
 			if( Write_fd_str( 2, msg ) < 0 ){
@@ -213,20 +210,20 @@ int Send_job( char *printer, char *host, struct control_file *cf,
 				exit(-2);
 			}
 		} else {
-			setstatus( NORMAL, msg );
+			setstatus( cfp, msg );
 		}
 	} else {
 		DEBUG8("Send_job: pc_used 0x%x, end %d, file %s, remote %d",
 			pc_used, Accounting_end, Accounting_file, Accounting_remote );
 		if( pc_used && Accounting_remote ){
-			i = Do_accounting( 1, Accounting_end, cf, Send_timeout, pc_used, 0 );
+			i = Do_accounting( 1, Accounting_end, cfp, Send_timeout, pc_used, 0 );
 			if( i ){
 				Errorcode = i;
 				cleanup( 0 );
 			}
 		}
-		setstatus( NORMAL, "done job '%s' transfer to %s@%s",
-			cf->name, printer, host );
+		setstatus( cfp, "done job '%s' transfer to %s@%s",
+			cfp->name, printer, host );
 	}
 	return( status );
 }
@@ -237,7 +234,7 @@ int Send_files(
 	char *printer,			- printer name
 	char *dpathname *dpath  - spool directory pathname
 	int *sock,					- socket to use
-	struct control_file *cf,	- control file
+	struct control_file *cfp,	- control file
 	int transfer_timeout,		- transfer timeout
 	)						- acknowlegement status
 
@@ -267,14 +264,14 @@ static char *cfbuf;
 
 int Send_files( char *host, char *printer,
 	struct dpathname *dpath,
-	int *sock, struct control_file *cf, int transfer_timeout,
+	int *sock, struct control_file *cfp, int transfer_timeout,
 	struct pc_used *pc_used )
 {
 	int status;			/* status of transfer command */
 	int size;			/* size of file */
 	char line[LINEBUFFER];	/* buffer for making up command to send */
-	int i, j;			/* ACME, the very finest in integer variables */
-	char *s;			/* AJAX, local pointers on demand */
+	int i, j, c;			/* ACME, the very finest in integer variables */
+	char *s, *t;			/* AJAX, local pointers on demand */
 	struct data_file *df;	/* data file entry */
 	struct stat statb;	/* status buffer */
 	int fd;				/* file descriptor */
@@ -283,13 +280,14 @@ int Send_files( char *host, char *printer,
 	int ack;			/* ack byte */
 	int err;			/* saved errno */
 	int sent_data = 0;		/* flag to handle sending data files first */
+	int fix_format;		/* fix up format */
 
 	DEBUG4("Send_files: send_data_first %d", Send_data_first );
-	setstatus( NORMAL, "sending '%s' to %s@%s", cf->name, printer, host );
+	setstatus( cfp, "sending '%s' to %s@%s", cfp->name, printer, host );
 	status = Link_send( host, sock, transfer_timeout,
 		REQ_RECV, printer, '\n', &ack );
 	if( status ){
-		setstatus( NORMAL, "error '%s' sending '\\%d%s\\n' to %s@%s",
+		setstatus( cfp, "error '%s' sending '\\%d%s\\n' to %s@%s",
 			Link_err_str(status), REQ_RECV, printer, printer, host );
 		return(status);
 	}
@@ -305,13 +303,13 @@ send_control:
 	 */
 
 	size = 0;
-	DEBUG4("Send_files: line_count %d", cf->info_lines.count );
+	DEBUG4("Send_files: line_count %d", cfp->info_lines.count );
 
-	lines = cf->info_lines.list;
+	lines = cfp->info_lines.list;
 	/* copy the lines into one big buffer */
 
 	j = 0;
-	for( i = 0; i < cf->info_lines.count; ++i ){
+	for( i = 0; i < cfp->info_lines.count; ++i ){
 		if( lines[i] && lines[i][0] ) j += strlen( lines[i] + 1 );
 	}
 	if( j >= cflen ){
@@ -324,11 +322,21 @@ send_control:
 	}
 	s = cfbuf;
 	*s = 0;
-	for( i = 0; i < cf->info_lines.count; ++i ){
-		if( lines[i] && lines[i][0] ){
+	fix_format = Bounce_queue_dest && !Interactive && Xlate_format;
+	for( i = 0; i < cfp->info_lines.count; ++i ){
+		if( lines[i] && (c = lines[i][0]) ){
 			s = s+strlen(s);
 			strcpy( s, lines[i] );
 			strcat( s, "\n" );
+			/* check to see if you need to translate formats */
+			if( fix_format && islower(c) && (t = strchr( Xlate_format, c )) ){
+				j = t - Xlate_format;
+				c = t[1];
+				if( (j & 1) == 0 && islower(c) ){
+					*s = c;
+					DEBUG4("Send_files: translate format '%s'", s );
+				}
+			}
 		}
 	}
 
@@ -339,7 +347,7 @@ send_control:
 	/*
 	 * send the command file name line
 	 */
-	plp_snprintf( line, sizeof(line), "%d %s", size, cf->name );
+	plp_snprintf( line, sizeof(line), "%d %s", size, cfp->name );
 	ack = 0;
 	errno = 0;
 	status = Link_send( host, sock, transfer_timeout,
@@ -362,7 +370,7 @@ send_control:
 	if( status != 0 ){
 		logerr( LOG_INFO,
 			"Send_files: error '%s' ack %d sending control file '%s' to %s@%s",
-			Link_err_str(status), ack, cf->name, printer, host );
+			Link_err_str(status), ack, cfp->name, printer, host );
 		switch( ack ){
 		case ACK_STOP_Q: status = JREMOVE; break;
 		default:         status = JFAIL; break;
@@ -371,9 +379,9 @@ send_control:
 	}
 	DEBUG4( "Send_files: control file sent, ack '%d'", ack );
 
-	DEBUG4( "Send_files: data file count '%d'", cf->data_file.count );
-	setstatus( NORMAL, "completed '%s' to %s@%s",
-		cf->name, printer, host );
+	DEBUG4( "Send_files: data file count '%d'", cfp->data_file.count );
+	setstatus( cfp, "completed '%s' to %s@%s",
+		cfp->name, printer, host );
 
 
 	/*
@@ -381,8 +389,8 @@ send_control:
 	 */
 
 send_data:
-	for( i = 0; sent_data == 0 && status == 0 && i < cf->data_file.count; ++i ){
-		df = (void *)cf->data_file.list;
+	for( i = 0; sent_data == 0 && status == 0 && i < cfp->data_file.count; ++i ){
+		df = (void *)cfp->data_file.list;
 		df = &df[i];
 		DEBUG4(
 	"Send_files: [%d] transfer '%s', open '%s', fd %d, size %d, copies %d",
@@ -393,7 +401,7 @@ send_data:
 			continue;
 		}
 		DEBUG4( "Send_files: file '%s'", df->openname );
-		setstatus( NORMAL, "sending '%s' to %s@%s",
+		setstatus( cfp, "sending '%s' to %s@%s",
 			df->transfername?df->transfername:df->openname,
 			printer, host );
 		fd = df->fd;
@@ -417,41 +425,41 @@ send_data:
 				char *s = strrchr( df->openname, '/' );
 				if( s == 0 ) s = df->openname;
 				/* we cannot open the data file! */
-				plp_snprintf( cf->error, sizeof(cf->error),
+				plp_snprintf( cfp->error, sizeof(cfp->error),
 					"cannot open '%s' - '%s'", s, Errormsg(err) );
-				logerr( LOG_ERR, "Send_files: %s", cf->error );
+				logerr( LOG_ERR, "Send_files: %s", cfp->error );
 				if( !Interactive ){
-					Set_job_control( cf );
+					Set_job_control( cfp );
 				}
-				return(JFAIL);
+				return(JREMOVE);
 			}
 			if( statb.st_dev != df->statb.st_dev ){
-				plp_snprintf( cf->error, sizeof(cf->error),
+				plp_snprintf( cfp->error, sizeof(cfp->error),
 					"file '%s', st_dev %d, not %d, possible security problem",
 					df->openname, statb.st_dev, df->statb.st_dev );
-				log( LOG_ERR, "Send_files: %s", cf->error );
+				log( LOG_ERR, "Send_files: %s", cfp->error );
 				if( !Interactive ){
-					Set_job_control( cf );
+					Set_job_control( cfp );
 				}
 				return(JREMOVE);
 			}
 			if( statb.st_ino != df->statb.st_ino ){
-				plp_snprintf( cf->error, sizeof(cf->error),
+				plp_snprintf( cfp->error, sizeof(cfp->error),
 				"file '%s', st_ino %d, not %d, possible security problem",
 					df->openname, statb.st_ino, df->statb.st_ino );
-				log( LOG_ERR, "Send_files: %s", cf->error );
+				log( LOG_ERR, "Send_files: %s", cfp->error );
 				if( !Interactive ){
-					Set_job_control( cf );
+					Set_job_control( cfp );
 				}
 				return(JREMOVE);
 			}
 			if( statb.st_size != df->statb.st_size ){
-				plp_snprintf( cf->error, sizeof(cf->error),
+				plp_snprintf( cfp->error, sizeof(cfp->error),
 					"file '%s', st_size %d, not %d, possible security problem",
 					df->openname, statb.st_size, df->statb.st_size );
-				log( LOG_ERR, "Send_files: %s", cf->error );
+				log( LOG_ERR, "Send_files: %s", cfp->error );
 				if( !Interactive ){
-					Set_job_control( cf );
+					Set_job_control( cfp );
 				}
 				return(JREMOVE);
 			}
@@ -460,12 +468,12 @@ send_data:
 			/* first we have to rewind the file */
 			if( lseek( fd, (off_t)0, 0 ) == (off_t)(-1) ){
 				err = errno;
-				plp_snprintf( cf->error, sizeof(cf->error),
+				plp_snprintf( cfp->error, sizeof(cfp->error),
 					"file '%s', fd %d, lseek failed - %s",
 					df->openname, fd, Errormsg(err) );
-				log( LOG_ERR, "Send_files: %s", cf->error );
+				log( LOG_ERR, "Send_files: %s", cfp->error );
 				if( !Interactive ){
-					Set_job_control( cf );
+					Set_job_control( cfp );
 				}
 				return(JREMOVE);
 			}
@@ -489,10 +497,15 @@ send_data:
 		 * temporary file,  and then use this as the real file.  When done,
 		 * we remove the temporary file.  To ensure that we do not
 		 * use too much space,  we will create only one temp file.
+		 *
+		 * Note: the 'p' format is supposed to be processed by the /bin/pr
+		 * program.  However,  it may also get processed again at the destination
+		 * by another filter.  We will assume that the user will provide a
+		 * special 'p' format filter or not have it filtered in a bounce queue.
 		 */
 
 		if( Bounce_queue_dest && !Interactive ){
-			if( (status = Bounce_filter( cf, df, pc_used, fd,
+			if( (status = Bounce_filter( cfp, df, pc_used, fd,
 					&size, &statb )) ){
 				return( status );
 			}
@@ -541,11 +554,11 @@ send_data:
 			log( LOG_INFO,
 				"Send_files: error '%s' sending data file '%s' to %s@%s",
 				Link_err_str( status ), df->openname, printer, host );
-			setstatus( NORMAL, "error sending '%s' to %s@%s",
+			setstatus( cfp, "error sending '%s' to %s@%s",
 				df->openname?df->openname:df->transfername,
-				cf->name, printer, host );
+				cfp->name, printer, host );
 		} else {
-			setstatus( NORMAL, "completed '%s' to %s@%s",
+			setstatus( cfp, "completed '%s' to %s@%s",
 				df->transfername?df->transfername:df->openname,
 				printer, host );
 		}
@@ -569,7 +582,6 @@ send_data:
  * 5. return the temp file size
  */
 
-static int tempfd = -1;
 
 static int Bounce_filter( struct control_file *cfp,
 	struct data_file *data_file, struct pc_used *pc_used,
@@ -578,53 +590,16 @@ static int Bounce_filter( struct control_file *cfp,
 	int c, in, err;
 	char *filter;
 	char tempbuf[LARGEBUFFER];
+	static int tempfd;
 
 	/* get the temp file */
-	if( tempfd < 0 ){
-		plp_snprintf( tempbuf, sizeof(tempbuf)-1, "bfA%03d%s", cfp->number, cfp->filehostname );
-		tempfd = Checkwrite( tempbuf, statb, O_RDWR, 1, 0 );
-		err = errno;
-		DEBUG4("Bounce_filter: temp file '%s', fd %d", tempbuf, tempfd );
-		if( tempfd < 0 ){
-			logerr_die( LOG_INFO,
-				"Bounce_filter: error '%s' opening bounce queue temp file '%s'",
-				Errormsg(err), tempbuf );
-			Errorcode = JFAIL;
-			cleanup( 0 );
-		}
-		/* leave the file if you need to debug it */
-		if( Debug < 8)(void)unlink( tempbuf );
-	}
-	if( ftruncate( tempfd, 0 ) < 0 ){
-		err = errno;
-		setstatus( NORMAL,
-			"Bounce_filter: error '%s' truncating bounce queue temp file '%s'",
-			Errormsg(err), tempbuf );
-		Errorcode = JFAIL;
-		cleanup( 0 );
-	}
-
-	/* fix up the control file 'Q' entry if necessary */
-
-	if( Use_queuename && cfp->QUEUENAME == 0 && Printer ){
-		static char buffer[M_QUEUENAME];
-		plp_snprintf( buffer, sizeof(buffer), "Q%s", Printer );
-		cfp->QUEUENAME = buffer;
-	}
-
 	/* now find the filter - similar to the printer support code */
 	filter = 0;
-	switch( (c = data_file->format) ){
-	case 'p':
-		if( Pr_program ){
-			static char pr_filter[SMALLBUFFER];
-			pr_filter[0] = 0;
-			safestrncat( pr_filter, Pr_program );
-			safestrncat( pr_filter, " -w%w -l%l" );
-			if( cfp->PRTITLE ) safestrncat( pr_filter, " -h %T" );
-			filter = pr_filter;
-		}
-		break;
+	/*
+	 * check for PR to be used
+	 */
+	c = data_file->format;
+	switch( c ){
 	case 'f': case 'l':
 		filter = IF_Filter;
 		break;
@@ -635,47 +610,65 @@ static int Bounce_filter( struct control_file *cfp,
 
 	/* hook filter up to the device file descriptor */
 	if( filter ){
+		if( tempfd == 0 ){
+			plp_snprintf( tempbuf, sizeof(tempbuf)-1,
+				"bfA%03d%s", cfp->number, cfp->filehostname );
+			tempfd = Checkwrite( tempbuf, statb, O_RDWR, 1, 0 );
+			err = errno;
+			DEBUG4("Bounce_filter: temp file '%s', fd %d", tempbuf, tempfd );
+			if( tempfd < 0 ){
+				Errorcode = JFAIL;
+				logerr_die( LOG_INFO,
+					"Bounce_filter: error '%s' opening bounce queue temp file '%s'",
+					Errormsg(err), tempbuf );
+			}
+			if( unlink( tempbuf ) < 0 ){
+				Errorcode = JFAIL;
+				logerr_die( LOG_INFO,
+					"Bounce_filter: error '%s' unlinking temp file '%s'",
+					Errormsg(err), tempbuf );
+			}
+		}
+		if( ftruncate( tempfd, 0 ) < 0 ){
+			err = errno;
+			Errorcode = JFAIL;
+			logerr_die( LOG_INFO,
+				"Bounce_filter: error '%s' truncating bounce queue temp fd",
+				Errormsg(err) );
+		}
 		Make_filter( c, cfp, &XF_fd_info, filter,
 			0, /* no extra */
 			0,	/* RW pipe */
 			tempfd, /* dup to fd 1 */
 			pc_used, /* printcap information */
-			data_file, Accounting_port );
+			data_file, Accounting_port, 0 );
 		/* at this point you have a filter, which is taking input
 			from XF_fd_info.input; pass input file through it */
 		while( (in = read(fd, tempbuf, sizeof(tempbuf)) ) > 0 ){
 			Write_fd_len( XF_fd_info.input, tempbuf, in );
 		}
 		if( Close_filter( &XF_fd_info, Send_timeout ) ){
+			err = errno;
 			Errorcode = JFAIL;
-			cleanup( 0 );
+			logerr_die( LOG_INFO,
+				"Bounce_filter: error '%s' closing bounce queue filter",
+				Errormsg(err) );
 		}
-		XF_fd_info.pid = 0;
-		XF_fd_info.input = -1;
 		/* now we close the input file and dup the filter output to it */
-		(void)close( fd );
 		if( dup2( tempfd, fd ) < 0 ){
 			Errorcode = JFAIL;
 			logerr_die( LOG_INFO, "Bounce_filter: dup2 failed" );
-			cleanup( 0 );
 		}
 		if( lseek( fd, (off_t)0, SEEK_SET ) < 0 ){
 			Errorcode = JFAIL;
 			logerr_die( LOG_INFO, "Bounce_filter: lseek failed" );
-			cleanup( 0 );
 		}
 		if( fstat( fd, statb ) < 0 ){
 			Errorcode = JFAIL;
 			logerr_die( LOG_INFO, "Bounce_filter: fstat failed" );
-			cleanup( 0 );
 		}
 		*size = statb->st_size;
 		DEBUG4("Bounce_filter: fd '%d', size %d", fd, *size );
-		if( lseek( fd, (off_t)0, SEEK_SET ) < 0 ){
-			Errorcode = JFAIL;
-			logerr_die( LOG_INFO, "Bounce_filter: lseek failed" );
-			cleanup( 0 );
-		}
 	}
 	return( 0 );
 }
