@@ -11,7 +11,7 @@
  **************************************************************************/
 
 static char *const _id =
-"$Id: setup_filter.c,v 3.3 1996/07/27 18:15:32 papowell Exp $";
+"$Id: setup_filter.c,v 3.5 1996/09/09 14:24:41 papowell Exp papowell $";
 
 /***************************************************************************
  *
@@ -69,6 +69,9 @@ static int cmd_split( struct filter *filter );
  *   int read_write          - create a read/write pipe
  *   int print_fd            - dup this fd to fd 1
  *   struct data_file *data_file   - data file being printed
+ *   int acct_port           - dup this fd to fd 3
+ *   int stderr_to_logger    - send the stderr status to a logger as well
+ *                           - as to stderr
  * If the command line is empty,  the printcap is searched for
  * a filter;  the command line is expanded
  *
@@ -88,9 +91,10 @@ int Make_filter( int key,
 	struct control_file *cf,
 	struct filter *filter, char *line, int noextra,
 	int read_write, int print_fd,
-	struct pc_used *pc_used, struct data_file *data_file, int acct_port )
+	struct pc_used *pc_used, struct data_file *data_file, int acct_port,
+	int stderr_to_logger )
 {
-	int p[2];
+	int p[2], stderr_pipe[2];
 	char *executable;
 	int root = 0;
 
@@ -143,9 +147,78 @@ int Make_filter( int key,
 	 * create pipe and fork
 	 */
 
-	DEBUG2 ("Make_filter: starting filter: %s", filter->cmd );
+	DEBUG2 ("Make_filter: stderr_to_logger %d, starting filter '%s'",
+		stderr_to_logger, filter->cmd );
 	if( (filter->pid = fork()) == 0 ){              /* child */
 		/* pipe is stdin */ /* printer is stdout */
+		if( stderr_to_logger ){
+			int len, count, i, filter_pid;
+			int pid;	/* for the stderr */
+			char *s, *end;
+			char stderr_header[SMALLBUFFER];
+			char stderr_info[SMALLBUFFER];
+
+			filter_pid = getpid();
+			if( pipe( stderr_pipe ) < 0 ){
+				Errorcode = JABORT;
+				logerr_die( LOG_ERR, "Make_filter: stderr_to_logger pipe failed" );
+			}
+			if( (pid = fork()) == 0 ){
+				/* we are the daughter, waiting for stderr output */
+				DEBUG8( "Make_filter: stderr pid %d, filter pid %d, pipe[%d,%d]",
+					getpid(), filter_pid, stderr_pipe[0], stderr_pipe[1] );
+				close( stderr_pipe[1] ); /* close stderr output side */
+				close( p[0] ); /* close filter input side */
+				close( p[1] ); /* close filter output side */
+				stderr_info[0] = 0;	/* clear out buffer */
+				stderr_info[1] = 0;	/* clear out buffer */
+				plp_snprintf( stderr_header, sizeof(stderr_header),
+					"FILTER_STATUS PID %d", filter_pid );
+				setmessage( cf, stderr_header, "COMMANDLINE %s", filter->cmd );
+				do{
+					/* we get the stderr information */
+					len = strlen( stderr_info+1 );
+					/* read more into buffer */
+					count = read( stderr_pipe[0], stderr_info+1+len, sizeof(stderr_info)-len-2 );
+					if( count > 0 ){
+						stderr_info[count+len+1] = 0;
+						/* we put out lines */
+						DEBUG9( "Make_filter: stderr pid read %d '%s'",
+							count, stderr_info+1 );
+						if( Write_fd_str( 2, stderr_info+1 ) < 0 ){
+							exit( 0 );
+						}
+						for( s = stderr_info+1;
+							s && (end = strchr( s, '\n' ));
+							s = end+1 ){
+							*end = 0;
+							if( *s == '.' ){
+								--s;
+								*s = '.';
+							}
+							setmessage( cf, stderr_header, "%s", s );
+						}
+						/* now keep the rest of the line */
+						if( s != stderr_info+1 ){
+							for( i = 0; (stderr_info[i+1] = s[i]); ++i );
+						}
+					}
+				}while( count > 0 );
+				s = stderr_info+1;
+				if( strlen( s ) ){
+					if( *s == '.' ){
+						--s;
+						*s = '.';
+					}
+					setmessage( cf, stderr_header, "%s", s );
+				}
+				setmessage( cf, stderr_header, "FINISHED" );
+				exit(0);
+			} else if( pid < 0 ){
+				Errorcode = errno;
+				logerr_die( LOG_NOTICE, "Make_filter: stderr_to_logger fork failed" );
+			}
+		}
 		if( dup2(p[0], 0) < 0 
 			|| (print_fd != 1 && dup2( print_fd, 1) < 0) ){
 			Errorcode = errno;
@@ -158,6 +231,14 @@ int Make_filter( int key,
 		} else
 #endif
 		Full_daemon_perms();   /* run as "daemon" */
+		if( stderr_to_logger ){
+			/* close input and dup output side to stderr */
+			close( stderr_pipe[0] );
+			if( dup2(stderr_pipe[1], 2) < 0  ){
+				Errorcode = errno;
+				logerr_die(LOG_NOTICE,"Make_filter: stderr_to_logger dup2 failed");
+			}
+		}
 		if( acct_port > 0 ){
 			dup2( acct_port, 3 );
 			setup_close_on_exec(4); /* this'll close the unused pipe fds */
@@ -524,8 +605,9 @@ do_dollar( struct control_file *cf, char *s, char *e, int type,
 static void setup_std_fds( void )
 {
     int fd;
-    while( (fd = open( "/dev/null", O_RDWR, Spool_file_perms)) >= 0 && fd < 3  ){
-        DEBUG9 ("setup_std_fds: opened /dev/null on fd %d", fd);
+    fd = open( "/dev/null", O_RDWR, Spool_file_perms);
+    while( fd >= 0 && fd < 3  ){
+		fd = dup( fd );
     }
     if( fd < 0 ){
         /* well, this is pretty serious */
@@ -736,7 +818,7 @@ char * find_executable( char *execname )
 		}
 
 		if( *step != '/' ){
-			logerr( LOG_ERR, "find_executable: Filter_path has bad entry '%s`",
+			logerr( LOG_ERR, "find_executable: Filter_path has bad entry '%s'",
 				Filter_path );
 		}
 
@@ -775,8 +857,8 @@ char * word_in_quotes( char *start )
 	for( end = start+1, begin = start;
 		(next = strchr( end, ch )); end = next+1 ){
 		len = (next - end);
-		DEBUG9("word_in_quotes: start 0x%x, begin 0x%x, next 0x%x, end 0x%x, len %d", 
-			start, begin, next, end, len );
+		/* DEBUG9("word_in_quotes: start 0x%x, begin 0x%x, next 0x%x, end 0x%x, len %d", 
+			start, begin, next, end, len ); */
 		strncpy( begin, end, len );
 		begin += len;
 		*begin = 0;
@@ -841,10 +923,10 @@ static int cmd_split( struct filter *filter )
 			filter->args.list[filter->args.count++] = start = cmd;
 
 			/* find end of word */
-			DEBUG9("cmd_split: starting '%s'", start );
+			/* DEBUG9("cmd_split: starting '%s'", start ); */
 			while( (c = *cmd) && !isspace( c ) ){
 				if( strchr( quotes, c ) ){
-					DEBUG9("cmd_split: starting quote '%s'", cmd );
+					/* DEBUG9("cmd_split: starting quote '%s'", cmd ); */
 					cmd = word_in_quotes( cmd );
 					if( cmd == 0 ){
 						logerr( LOG_ERR, "cmd_split: unbalanced quote in '%s`",
@@ -862,7 +944,7 @@ static int cmd_split( struct filter *filter )
 			if( cmd && *cmd ){
 				*cmd++ = 0;
 			}
-			DEBUG9("cmd_split: word '%s'", start );
+			/* DEBUG9("cmd_split: word '%s'", start ); */
 		}
 	}
 	if( filter->args.count >= filter->args.max ){
@@ -988,6 +1070,6 @@ int Close_filter( struct filter *filter, int timeout )
 				Server_status( status ) );
 		}
 	}
-	DEBUG5("Close_filter: status 0x%xd", status );
+	DEBUG5("Close_filter: status 0x%x", status );
 	return( status );
 }
