@@ -8,7 +8,7 @@
  ***************************************************************************
  * MODULE: lp.h
  * PURPOSE: general type definitions that are used by all LPX facilities.
- * $Id: lp.h,v 3.1 1996/06/30 17:12:44 papowell Exp $
+ * $Id: lp.h,v 3.3 1996/08/25 22:20:05 papowell Exp papowell $
  **************************************************************************/
 
 #ifndef _LP_H_
@@ -451,10 +451,14 @@ struct control_file {
 	long priority_time;		/* priority time */
 	long hold_time;			/* hold time */
 	long remove_time;		/* removal time */
+	long done_time;			/* finished time */
 	long move_time;			/* time when moved */
+	long routed_time;		/* routed to destinations */
 	char *hold_file;		/* pathname of hold file */
 	char *spool_dir;		/* pathname of spool dir */
 	char *control_dir;		/* pathname of control dir */
+	char *identifier;	/* identifier */
+	char ident_buffer[LINEBUFFER];	/* identifier */
 	char redirect[LINEBUFFER];	/* redirection */
 	char error[LINEBUFFER];	/* error message for bad control file */
 	int print_attempts;		/* number of print attempts */
@@ -469,6 +473,25 @@ struct control_file {
 											and name */
 	struct malloc_list data_file;		/* allocated area for data file list */
 	struct malloc_list info_lines;		/* allocated area for line list */
+	struct malloc_list status_file;		/* allocated area for status file */
+	struct malloc_list destination_lines;	/* allocated area for destination lines */
+	struct malloc_list destination_list;	/* allocated area for destination list */
+};
+
+struct destination {
+	char destination[LINEBUFFER];	/* destination of job */
+	char identifier[LINEBUFFER];	/* new identifier of job */
+	char error[LINEBUFFER];			/* error message for bad control file */
+	int arg_start;					/* option lines to add/change */
+	int arg_count;					/* number of lines */
+	int copies;						/* number of copies */
+	int copy_done;					/* copies done */
+	int status;						/* status of copy */
+	int hold;						/* held */
+	int active;						/* active */
+	int done;						/* active */
+	int attempt;					/* attempt */
+	int ignore;						/* ignore */
 };
 
 int Parse_cf( struct dpathname *dpath,
@@ -504,6 +527,8 @@ void Clear_control_file( struct control_file *cf );
  */
 
 #define M_DEFAULT		131				/* default limit on line length */
+#define IDENTIFIER capoptions['A'-'A']	/* LPRng - unique job ID */
+#define M_IDENTIFIER	131				/* default limit on line length */
 #define CLASSNAME capoptions['C'-'A']	/* RFC: 31 char limit */
 #define M_CLASSNAME		31
 #define FROMHOST  capoptions['H'-'A']	/* RFC: 31 char limit */
@@ -559,6 +584,7 @@ int Link_getreuse( int sock );
 int Link_dest_port_num( void );
 int Link_listen(void);
 int Link_open(char *host, int retry, int timeout );
+int Link_open_type(char *host, int retry, int timeout, int port, int connection_type );
 void Link_close( int *sock );
 int Link_ack( char *host, int *socket, int timeout, int sendc, int *ack );
 int Link_send ( char *host, int *socket, int timeout,
@@ -591,12 +617,15 @@ const char *Ack_err_str (int n);
  *  a log file depending on system that is using it.
  *****************************************************************/
 
-enum statuskind { NORMAL, ERROR };
+#define NORMAL (0)
 #ifdef HAVE_STDARGS
-void setstatus( int kind, char *fmt, ... );
+void setstatus( struct control_file *cfp, char *fmt, ... );
+void setmessage( struct control_file *cfp, char *msg, char *fmt, ... );
 #else
 void setstatus( va_alist );
+void setmessage( va_alist );
 #endif
+void send_to_logger( char *msg );
 
 
 /*****************************************************************
@@ -747,8 +776,12 @@ EXTERN char *RemoteHost;	/* Remote Host */
 EXTERN char *RemotePrinter;	/* Remote Printer */
 EXTERN int Status_fd;		/* status file file descriptor */
 
-/* for LPR use */
-EXTERN int Use_shorthost;	/* Use short hostname in control file information */
+/*****************************************************************
+ * Routing Support
+ *****************************************************************/
+
+EXTERN struct destination *Destination;	/* Destination for routing */
+
 
 #include "errormsg.h"
 
@@ -756,6 +789,8 @@ EXTERN int Use_shorthost;	/* Use short hostname in control file information */
 	Malloc_failed((unsigned)y); \
 }
 
+#define roundup_2(x,y) ((x+((1<<y)-1))&~((1<<y)-1))
+
 /*****************************************************************
  * Handy utilty functions: write (bombproof) to a file descriptor
  *****************************************************************/
